package com.emerald.rcm.backend.repository;

import com.emerald.rcm.backend.domain.SecurityControl;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.stream.IntStream;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;

/**
 * Utility repository to load bag relationships based on https://vladmihalcea.com/hibernate-multiplebagfetchexception/
 */
public class SecurityControlRepositoryWithBagRelationshipsImpl implements SecurityControlRepositoryWithBagRelationships {

    private static final String ID_PARAMETER = "id";
    private static final String SECURITYCONTROLS_PARAMETER = "securityControls";

    @PersistenceContext
    private EntityManager entityManager;

    @Override
    public Optional<SecurityControl> fetchBagRelationships(Optional<SecurityControl> securityControl) {
        return securityControl.map(this::fetchResources);
    }

    @Override
    public Page<SecurityControl> fetchBagRelationships(Page<SecurityControl> securityControls) {
        return new PageImpl<>(
            fetchBagRelationships(securityControls.getContent()),
            securityControls.getPageable(),
            securityControls.getTotalElements()
        );
    }

    @Override
    public List<SecurityControl> fetchBagRelationships(List<SecurityControl> securityControls) {
        return Optional.of(securityControls).map(this::fetchResources).orElse(Collections.emptyList());
    }

    SecurityControl fetchResources(SecurityControl result) {
        return entityManager
            .createQuery(
                "select securityControl from SecurityControl securityControl left join fetch securityControl.resources where securityControl.id = :id",
                SecurityControl.class
            )
            .setParameter(ID_PARAMETER, result.getId())
            .getSingleResult();
    }

    List<SecurityControl> fetchResources(List<SecurityControl> securityControls) {
        HashMap<Object, Integer> order = new HashMap<>();
        IntStream.range(0, securityControls.size()).forEach(index -> order.put(securityControls.get(index).getId(), index));
        List<SecurityControl> result = entityManager
            .createQuery(
                "select securityControl from SecurityControl securityControl left join fetch securityControl.resources where securityControl in :securityControls",
                SecurityControl.class
            )
            .setParameter(SECURITYCONTROLS_PARAMETER, securityControls)
            .getResultList();
        Collections.sort(result, (o1, o2) -> Integer.compare(order.get(o1.getId()), order.get(o2.getId())));
        return result;
    }
}
