package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.CloudServiceProvider;
import com.emerald.rcm.backend.repository.CloudServiceProviderRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.CloudServiceProvider}.
 */
@RestController
@RequestMapping("/api/cloud-service-providers")
@Transactional
public class CloudServiceProviderResource {

    private static final Logger log = LoggerFactory.getLogger(CloudServiceProviderResource.class);

    private static final String ENTITY_NAME = "rcmBackendCloudServiceProvider";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final CloudServiceProviderRepository cloudServiceProviderRepository;

    public CloudServiceProviderResource(CloudServiceProviderRepository cloudServiceProviderRepository) {
        this.cloudServiceProviderRepository = cloudServiceProviderRepository;
    }

    /**
     * {@code POST  /cloud-service-providers} : Create a new cloudServiceProvider.
     *
     * @param cloudServiceProvider the cloudServiceProvider to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new cloudServiceProvider, or with status {@code 400 (Bad Request)} if the cloudServiceProvider has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<CloudServiceProvider> createCloudServiceProvider(@Valid @RequestBody CloudServiceProvider cloudServiceProvider)
        throws URISyntaxException {
        log.debug("REST request to save CloudServiceProvider : {}", cloudServiceProvider);
        if (cloudServiceProvider.getId() != null) {
            throw new BadRequestAlertException("A new cloudServiceProvider cannot already have an ID", ENTITY_NAME, "idexists");
        }
        cloudServiceProvider = cloudServiceProviderRepository.save(cloudServiceProvider);
        return ResponseEntity.created(new URI("/api/cloud-service-providers/" + cloudServiceProvider.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, cloudServiceProvider.getId().toString()))
            .body(cloudServiceProvider);
    }

    /**
     * {@code PUT  /cloud-service-providers/:id} : Updates an existing cloudServiceProvider.
     *
     * @param id the id of the cloudServiceProvider to save.
     * @param cloudServiceProvider the cloudServiceProvider to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated cloudServiceProvider,
     * or with status {@code 400 (Bad Request)} if the cloudServiceProvider is not valid,
     * or with status {@code 500 (Internal Server Error)} if the cloudServiceProvider couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<CloudServiceProvider> updateCloudServiceProvider(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody CloudServiceProvider cloudServiceProvider
    ) throws URISyntaxException {
        log.debug("REST request to update CloudServiceProvider : {}, {}", id, cloudServiceProvider);
        if (cloudServiceProvider.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, cloudServiceProvider.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!cloudServiceProviderRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        cloudServiceProvider = cloudServiceProviderRepository.save(cloudServiceProvider);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, cloudServiceProvider.getId().toString()))
            .body(cloudServiceProvider);
    }

    /**
     * {@code PATCH  /cloud-service-providers/:id} : Partial updates given fields of an existing cloudServiceProvider, field will ignore if it is null
     *
     * @param id the id of the cloudServiceProvider to save.
     * @param cloudServiceProvider the cloudServiceProvider to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated cloudServiceProvider,
     * or with status {@code 400 (Bad Request)} if the cloudServiceProvider is not valid,
     * or with status {@code 404 (Not Found)} if the cloudServiceProvider is not found,
     * or with status {@code 500 (Internal Server Error)} if the cloudServiceProvider couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<CloudServiceProvider> partialUpdateCloudServiceProvider(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody CloudServiceProvider cloudServiceProvider
    ) throws URISyntaxException {
        log.debug("REST request to partial update CloudServiceProvider partially : {}, {}", id, cloudServiceProvider);
        if (cloudServiceProvider.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, cloudServiceProvider.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!cloudServiceProviderRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<CloudServiceProvider> result = cloudServiceProviderRepository
            .findById(cloudServiceProvider.getId())
            .map(existingCloudServiceProvider -> {
                if (cloudServiceProvider.getAcronym() != null) {
                    existingCloudServiceProvider.setAcronym(cloudServiceProvider.getAcronym());
                }
                if (cloudServiceProvider.getName() != null) {
                    existingCloudServiceProvider.setName(cloudServiceProvider.getName());
                }
                if (cloudServiceProvider.getContactDetails() != null) {
                    existingCloudServiceProvider.setContactDetails(cloudServiceProvider.getContactDetails());
                }

                return existingCloudServiceProvider;
            })
            .map(cloudServiceProviderRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, cloudServiceProvider.getId().toString())
        );
    }

    /**
     * {@code GET  /cloud-service-providers} : get all the cloudServiceProviders.
     *
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of cloudServiceProviders in body.
     */
    @GetMapping("")
    public List<CloudServiceProvider> getAllCloudServiceProviders() {
        log.debug("REST request to get all CloudServiceProviders");
        return cloudServiceProviderRepository.findAll();
    }

    /**
     * {@code GET  /cloud-service-providers/:id} : get the "id" cloudServiceProvider.
     *
     * @param id the id of the cloudServiceProvider to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the cloudServiceProvider, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<CloudServiceProvider> getCloudServiceProvider(@PathVariable("id") Long id) {
        log.debug("REST request to get CloudServiceProvider : {}", id);
        Optional<CloudServiceProvider> cloudServiceProvider = cloudServiceProviderRepository.findById(id);
        return ResponseUtil.wrapOrNotFound(cloudServiceProvider);
    }

    /**
     * {@code DELETE  /cloud-service-providers/:id} : delete the "id" cloudServiceProvider.
     *
     * @param id the id of the cloudServiceProvider to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteCloudServiceProvider(@PathVariable("id") Long id) {
        log.debug("REST request to delete CloudServiceProvider : {}", id);
        cloudServiceProviderRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
