package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.CloudService;
import com.emerald.rcm.backend.repository.CloudServiceRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.CloudService}.
 */
@RestController
@RequestMapping("/api/cloud-services")
@Transactional
public class CloudServiceResource {

    private static final Logger log = LoggerFactory.getLogger(CloudServiceResource.class);

    private static final String ENTITY_NAME = "rcmBackendCloudService";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final CloudServiceRepository cloudServiceRepository;

    public CloudServiceResource(CloudServiceRepository cloudServiceRepository) {
        this.cloudServiceRepository = cloudServiceRepository;
    }

    /**
     * {@code POST  /cloud-services} : Create a new cloudService.
     *
     * @param cloudService the cloudService to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new cloudService, or with status {@code 400 (Bad Request)} if the cloudService has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<CloudService> createCloudService(@Valid @RequestBody CloudService cloudService) throws URISyntaxException {
        log.debug("REST request to save CloudService : {}", cloudService);
        if (cloudService.getId() != null) {
            throw new BadRequestAlertException("A new cloudService cannot already have an ID", ENTITY_NAME, "idexists");
        }
        cloudService = cloudServiceRepository.save(cloudService);
        return ResponseEntity.created(new URI("/api/cloud-services/" + cloudService.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, cloudService.getId().toString()))
            .body(cloudService);
    }

    /**
     * {@code PUT  /cloud-services/:id} : Updates an existing cloudService.
     *
     * @param id the id of the cloudService to save.
     * @param cloudService the cloudService to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated cloudService,
     * or with status {@code 400 (Bad Request)} if the cloudService is not valid,
     * or with status {@code 500 (Internal Server Error)} if the cloudService couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<CloudService> updateCloudService(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody CloudService cloudService
    ) throws URISyntaxException {
        log.debug("REST request to update CloudService : {}, {}", id, cloudService);
        if (cloudService.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, cloudService.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!cloudServiceRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        cloudService = cloudServiceRepository.save(cloudService);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, cloudService.getId().toString()))
            .body(cloudService);
    }

    /**
     * {@code PATCH  /cloud-services/:id} : Partial updates given fields of an existing cloudService, field will ignore if it is null
     *
     * @param id the id of the cloudService to save.
     * @param cloudService the cloudService to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated cloudService,
     * or with status {@code 400 (Bad Request)} if the cloudService is not valid,
     * or with status {@code 404 (Not Found)} if the cloudService is not found,
     * or with status {@code 500 (Internal Server Error)} if the cloudService couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<CloudService> partialUpdateCloudService(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody CloudService cloudService
    ) throws URISyntaxException {
        log.debug("REST request to partial update CloudService partially : {}, {}", id, cloudService);
        if (cloudService.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, cloudService.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!cloudServiceRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<CloudService> result = cloudServiceRepository
            .findById(cloudService.getId())
            .map(existingCloudService -> {
                if (cloudService.getAcronym() != null) {
                    existingCloudService.setAcronym(cloudService.getAcronym());
                }
                if (cloudService.getName() != null) {
                    existingCloudService.setName(cloudService.getName());
                }
                if (cloudService.getType() != null) {
                    existingCloudService.setType(cloudService.getType());
                }
                if (cloudService.getDescription() != null) {
                    existingCloudService.setDescription(cloudService.getDescription());
                }

                return existingCloudService;
            })
            .map(cloudServiceRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, cloudService.getId().toString())
        );
    }

    /**
     * {@code GET  /cloud-services} : get all the cloudServices.
     *
     * @param eagerload flag to eager load entities from relationships (This is applicable for many-to-many).
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of cloudServices in body.
     */
    @GetMapping("")
    public List<CloudService> getAllCloudServices(
        @RequestParam(name = "eagerload", required = false, defaultValue = "true") boolean eagerload
    ) {
        log.debug("REST request to get all CloudServices");
        if (eagerload) {
            return cloudServiceRepository.findAllWithEagerRelationships();
        } else {
            return cloudServiceRepository.findAll();
        }
    }

    /**
     * {@code GET  /cloud-services/:id} : get the "id" cloudService.
     *
     * @param id the id of the cloudService to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the cloudService, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<CloudService> getCloudService(@PathVariable("id") Long id) {
        log.debug("REST request to get CloudService : {}", id);
        Optional<CloudService> cloudService = cloudServiceRepository.findOneWithEagerRelationships(id);
        return ResponseUtil.wrapOrNotFound(cloudService);
    }

    /**
     * {@code DELETE  /cloud-services/:id} : delete the "id" cloudService.
     *
     * @param id the id of the cloudService to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteCloudService(@PathVariable("id") Long id) {
        log.debug("REST request to delete CloudService : {}", id);
        cloudServiceRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
