package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.QuestionAssuranceLevel;
import com.emerald.rcm.backend.repository.QuestionAssuranceLevelRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.QuestionAssuranceLevel}.
 */
@RestController
@RequestMapping("/api/question-assurance-levels")
@Transactional
public class QuestionAssuranceLevelResource {

    private static final Logger log = LoggerFactory.getLogger(QuestionAssuranceLevelResource.class);

    private static final String ENTITY_NAME = "rcmBackendQuestionAssuranceLevel";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final QuestionAssuranceLevelRepository questionAssuranceLevelRepository;

    public QuestionAssuranceLevelResource(QuestionAssuranceLevelRepository questionAssuranceLevelRepository) {
        this.questionAssuranceLevelRepository = questionAssuranceLevelRepository;
    }

    /**
     * {@code POST  /question-assurance-levels} : Create a new questionAssuranceLevel.
     *
     * @param questionAssuranceLevel the questionAssuranceLevel to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new questionAssuranceLevel, or with status {@code 400 (Bad Request)} if the questionAssuranceLevel has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<QuestionAssuranceLevel> createQuestionAssuranceLevel(
        @Valid @RequestBody QuestionAssuranceLevel questionAssuranceLevel
    ) throws URISyntaxException {
        log.debug("REST request to save QuestionAssuranceLevel : {}", questionAssuranceLevel);
        if (questionAssuranceLevel.getId() != null) {
            throw new BadRequestAlertException("A new questionAssuranceLevel cannot already have an ID", ENTITY_NAME, "idexists");
        }
        questionAssuranceLevel = questionAssuranceLevelRepository.save(questionAssuranceLevel);
        return ResponseEntity.created(new URI("/api/question-assurance-levels/" + questionAssuranceLevel.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, questionAssuranceLevel.getId().toString()))
            .body(questionAssuranceLevel);
    }

    /**
     * {@code PUT  /question-assurance-levels/:id} : Updates an existing questionAssuranceLevel.
     *
     * @param id the id of the questionAssuranceLevel to save.
     * @param questionAssuranceLevel the questionAssuranceLevel to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated questionAssuranceLevel,
     * or with status {@code 400 (Bad Request)} if the questionAssuranceLevel is not valid,
     * or with status {@code 500 (Internal Server Error)} if the questionAssuranceLevel couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<QuestionAssuranceLevel> updateQuestionAssuranceLevel(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody QuestionAssuranceLevel questionAssuranceLevel
    ) throws URISyntaxException {
        log.debug("REST request to update QuestionAssuranceLevel : {}, {}", id, questionAssuranceLevel);
        if (questionAssuranceLevel.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, questionAssuranceLevel.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!questionAssuranceLevelRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        questionAssuranceLevel = questionAssuranceLevelRepository.save(questionAssuranceLevel);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, questionAssuranceLevel.getId().toString()))
            .body(questionAssuranceLevel);
    }

    /**
     * {@code PATCH  /question-assurance-levels/:id} : Partial updates given fields of an existing questionAssuranceLevel, field will ignore if it is null
     *
     * @param id the id of the questionAssuranceLevel to save.
     * @param questionAssuranceLevel the questionAssuranceLevel to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated questionAssuranceLevel,
     * or with status {@code 400 (Bad Request)} if the questionAssuranceLevel is not valid,
     * or with status {@code 404 (Not Found)} if the questionAssuranceLevel is not found,
     * or with status {@code 500 (Internal Server Error)} if the questionAssuranceLevel couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<QuestionAssuranceLevel> partialUpdateQuestionAssuranceLevel(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody QuestionAssuranceLevel questionAssuranceLevel
    ) throws URISyntaxException {
        log.debug("REST request to partial update QuestionAssuranceLevel partially : {}, {}", id, questionAssuranceLevel);
        if (questionAssuranceLevel.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, questionAssuranceLevel.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!questionAssuranceLevelRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<QuestionAssuranceLevel> result = questionAssuranceLevelRepository
            .findById(questionAssuranceLevel.getId())
            .map(existingQuestionAssuranceLevel -> {
                if (questionAssuranceLevel.getAssuranceLevel() != null) {
                    existingQuestionAssuranceLevel.setAssuranceLevel(questionAssuranceLevel.getAssuranceLevel());
                }

                return existingQuestionAssuranceLevel;
            })
            .map(questionAssuranceLevelRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, questionAssuranceLevel.getId().toString())
        );
    }

    /**
     * {@code GET  /question-assurance-levels} : get all the questionAssuranceLevels.
     *
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of questionAssuranceLevels in body.
     */
    @GetMapping("")
    public List<QuestionAssuranceLevel> getAllQuestionAssuranceLevels() {
        log.debug("REST request to get all QuestionAssuranceLevels");
        return questionAssuranceLevelRepository.findAll();
    }

    /**
     * {@code GET  /question-assurance-levels/:id} : get the "id" questionAssuranceLevel.
     *
     * @param id the id of the questionAssuranceLevel to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the questionAssuranceLevel, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<QuestionAssuranceLevel> getQuestionAssuranceLevel(@PathVariable("id") Long id) {
        log.debug("REST request to get QuestionAssuranceLevel : {}", id);
        Optional<QuestionAssuranceLevel> questionAssuranceLevel = questionAssuranceLevelRepository.findById(id);
        return ResponseUtil.wrapOrNotFound(questionAssuranceLevel);
    }

    /**
     * {@code DELETE  /question-assurance-levels/:id} : delete the "id" questionAssuranceLevel.
     *
     * @param id the id of the questionAssuranceLevel to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteQuestionAssuranceLevel(@PathVariable("id") Long id) {
        log.debug("REST request to delete QuestionAssuranceLevel : {}", id);
        questionAssuranceLevelRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
