package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.QuestionnaireNonConformity;
import com.emerald.rcm.backend.repository.QuestionnaireNonConformityRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.QuestionnaireNonConformity}.
 */
@RestController
@RequestMapping("/api/questionnaire-non-conformities")
@Transactional
public class QuestionnaireNonConformityResource {

    private static final Logger log = LoggerFactory.getLogger(QuestionnaireNonConformityResource.class);

    private static final String ENTITY_NAME = "rcmBackendQuestionnaireNonConformity";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final QuestionnaireNonConformityRepository questionnaireNonConformityRepository;

    public QuestionnaireNonConformityResource(QuestionnaireNonConformityRepository questionnaireNonConformityRepository) {
        this.questionnaireNonConformityRepository = questionnaireNonConformityRepository;
    }

    /**
     * {@code POST  /questionnaire-non-conformities} : Create a new questionnaireNonConformity.
     *
     * @param questionnaireNonConformity the questionnaireNonConformity to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new questionnaireNonConformity, or with status {@code 400 (Bad Request)} if the questionnaireNonConformity has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<QuestionnaireNonConformity> createQuestionnaireNonConformity(
        @Valid @RequestBody QuestionnaireNonConformity questionnaireNonConformity
    ) throws URISyntaxException {
        log.debug("REST request to save QuestionnaireNonConformity : {}", questionnaireNonConformity);
        if (questionnaireNonConformity.getId() != null) {
            throw new BadRequestAlertException("A new questionnaireNonConformity cannot already have an ID", ENTITY_NAME, "idexists");
        }
        questionnaireNonConformity = questionnaireNonConformityRepository.save(questionnaireNonConformity);
        return ResponseEntity.created(new URI("/api/questionnaire-non-conformities/" + questionnaireNonConformity.getId()))
            .headers(
                HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, questionnaireNonConformity.getId().toString())
            )
            .body(questionnaireNonConformity);
    }

    /**
     * {@code PUT  /questionnaire-non-conformities/:id} : Updates an existing questionnaireNonConformity.
     *
     * @param id the id of the questionnaireNonConformity to save.
     * @param questionnaireNonConformity the questionnaireNonConformity to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated questionnaireNonConformity,
     * or with status {@code 400 (Bad Request)} if the questionnaireNonConformity is not valid,
     * or with status {@code 500 (Internal Server Error)} if the questionnaireNonConformity couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<QuestionnaireNonConformity> updateQuestionnaireNonConformity(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody QuestionnaireNonConformity questionnaireNonConformity
    ) throws URISyntaxException {
        log.debug("REST request to update QuestionnaireNonConformity : {}, {}", id, questionnaireNonConformity);
        if (questionnaireNonConformity.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, questionnaireNonConformity.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!questionnaireNonConformityRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        questionnaireNonConformity = questionnaireNonConformityRepository.save(questionnaireNonConformity);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, questionnaireNonConformity.getId().toString()))
            .body(questionnaireNonConformity);
    }

    /**
     * {@code PATCH  /questionnaire-non-conformities/:id} : Partial updates given fields of an existing questionnaireNonConformity, field will ignore if it is null
     *
     * @param id the id of the questionnaireNonConformity to save.
     * @param questionnaireNonConformity the questionnaireNonConformity to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated questionnaireNonConformity,
     * or with status {@code 400 (Bad Request)} if the questionnaireNonConformity is not valid,
     * or with status {@code 404 (Not Found)} if the questionnaireNonConformity is not found,
     * or with status {@code 500 (Internal Server Error)} if the questionnaireNonConformity couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<QuestionnaireNonConformity> partialUpdateQuestionnaireNonConformity(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody QuestionnaireNonConformity questionnaireNonConformity
    ) throws URISyntaxException {
        log.debug("REST request to partial update QuestionnaireNonConformity partially : {}, {}", id, questionnaireNonConformity);
        if (questionnaireNonConformity.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, questionnaireNonConformity.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!questionnaireNonConformityRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<QuestionnaireNonConformity> result = questionnaireNonConformityRepository
            .findById(questionnaireNonConformity.getId())
            .map(existingQuestionnaireNonConformity -> {
                if (questionnaireNonConformity.getQuestionnaireName() != null) {
                    existingQuestionnaireNonConformity.setQuestionnaireName(questionnaireNonConformity.getQuestionnaireName());
                }
                if (questionnaireNonConformity.getComments() != null) {
                    existingQuestionnaireNonConformity.setComments(questionnaireNonConformity.getComments());
                }
                if (questionnaireNonConformity.getCompliance() != null) {
                    existingQuestionnaireNonConformity.setCompliance(questionnaireNonConformity.getCompliance());
                }

                return existingQuestionnaireNonConformity;
            })
            .map(questionnaireNonConformityRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, questionnaireNonConformity.getId().toString())
        );
    }

    /**
     * {@code GET  /questionnaire-non-conformities} : get all the questionnaireNonConformities.
     *
     * @param eagerload flag to eager load entities from relationships (This is applicable for many-to-many).
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of questionnaireNonConformities in body.
     */
    @GetMapping("")
    public List<QuestionnaireNonConformity> getAllQuestionnaireNonConformities(
        @RequestParam(name = "eagerload", required = false, defaultValue = "true") boolean eagerload
    ) {
        log.debug("REST request to get all QuestionnaireNonConformities");
        if (eagerload) {
            return questionnaireNonConformityRepository.findAllWithEagerRelationships();
        } else {
            return questionnaireNonConformityRepository.findAll();
        }
    }

    /**
     * {@code GET  /questionnaire-non-conformities/:id} : get the "id" questionnaireNonConformity.
     *
     * @param id the id of the questionnaireNonConformity to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the questionnaireNonConformity, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<QuestionnaireNonConformity> getQuestionnaireNonConformity(@PathVariable("id") Long id) {
        log.debug("REST request to get QuestionnaireNonConformity : {}", id);
        Optional<QuestionnaireNonConformity> questionnaireNonConformity =
            questionnaireNonConformityRepository.findOneWithEagerRelationships(id);
        return ResponseUtil.wrapOrNotFound(questionnaireNonConformity);
    }

    /**
     * {@code DELETE  /questionnaire-non-conformities/:id} : delete the "id" questionnaireNonConformity.
     *
     * @param id the id of the questionnaireNonConformity to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteQuestionnaireNonConformity(@PathVariable("id") Long id) {
        log.debug("REST request to delete QuestionnaireNonConformity : {}", id);
        questionnaireNonConformityRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
