package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.ReferenceTom;
import com.emerald.rcm.backend.repository.ReferenceTomRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.ReferenceTom}.
 */
@RestController
@RequestMapping("/api/reference-toms")
@Transactional
public class ReferenceTomResource {

    private static final Logger log = LoggerFactory.getLogger(ReferenceTomResource.class);

    private static final String ENTITY_NAME = "rcmBackendReferenceTom";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final ReferenceTomRepository referenceTomRepository;

    public ReferenceTomResource(ReferenceTomRepository referenceTomRepository) {
        this.referenceTomRepository = referenceTomRepository;
    }

    /**
     * {@code POST  /reference-toms} : Create a new referenceTom.
     *
     * @param referenceTom the referenceTom to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new referenceTom, or with status {@code 400 (Bad Request)} if the referenceTom has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<ReferenceTom> createReferenceTom(@Valid @RequestBody ReferenceTom referenceTom) throws URISyntaxException {
        log.debug("REST request to save ReferenceTom : {}", referenceTom);
        if (referenceTom.getId() != null) {
            throw new BadRequestAlertException("A new referenceTom cannot already have an ID", ENTITY_NAME, "idexists");
        }
        referenceTom = referenceTomRepository.save(referenceTom);
        return ResponseEntity.created(new URI("/api/reference-toms/" + referenceTom.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, referenceTom.getId().toString()))
            .body(referenceTom);
    }

    /**
     * {@code PUT  /reference-toms/:id} : Updates an existing referenceTom.
     *
     * @param id the id of the referenceTom to save.
     * @param referenceTom the referenceTom to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated referenceTom,
     * or with status {@code 400 (Bad Request)} if the referenceTom is not valid,
     * or with status {@code 500 (Internal Server Error)} if the referenceTom couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<ReferenceTom> updateReferenceTom(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody ReferenceTom referenceTom
    ) throws URISyntaxException {
        log.debug("REST request to update ReferenceTom : {}, {}", id, referenceTom);
        if (referenceTom.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, referenceTom.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!referenceTomRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        referenceTom = referenceTomRepository.save(referenceTom);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, referenceTom.getId().toString()))
            .body(referenceTom);
    }

    /**
     * {@code PATCH  /reference-toms/:id} : Partial updates given fields of an existing referenceTom, field will ignore if it is null
     *
     * @param id the id of the referenceTom to save.
     * @param referenceTom the referenceTom to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated referenceTom,
     * or with status {@code 400 (Bad Request)} if the referenceTom is not valid,
     * or with status {@code 404 (Not Found)} if the referenceTom is not found,
     * or with status {@code 500 (Internal Server Error)} if the referenceTom couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<ReferenceTom> partialUpdateReferenceTom(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody ReferenceTom referenceTom
    ) throws URISyntaxException {
        log.debug("REST request to partial update ReferenceTom partially : {}, {}", id, referenceTom);
        if (referenceTom.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, referenceTom.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!referenceTomRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<ReferenceTom> result = referenceTomRepository
            .findById(referenceTom.getId())
            .map(existingReferenceTom -> {
                if (referenceTom.getReferenceTomImpl() != null) {
                    existingReferenceTom.setReferenceTomImpl(referenceTom.getReferenceTomImpl());
                }

                return existingReferenceTom;
            })
            .map(referenceTomRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, referenceTom.getId().toString())
        );
    }

    /**
     * {@code GET  /reference-toms} : get all the referenceToms.
     *
     * @param eagerload flag to eager load entities from relationships (This is applicable for many-to-many).
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of referenceToms in body.
     */
    @GetMapping("")
    public List<ReferenceTom> getAllReferenceToms(
        @RequestParam(name = "eagerload", required = false, defaultValue = "true") boolean eagerload
    ) {
        log.debug("REST request to get all ReferenceToms");
        if (eagerload) {
            return referenceTomRepository.findAllWithEagerRelationships();
        } else {
            return referenceTomRepository.findAll();
        }
    }

    /**
     * {@code GET  /reference-toms/:id} : get the "id" referenceTom.
     *
     * @param id the id of the referenceTom to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the referenceTom, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<ReferenceTom> getReferenceTom(@PathVariable("id") Long id) {
        log.debug("REST request to get ReferenceTom : {}", id);
        Optional<ReferenceTom> referenceTom = referenceTomRepository.findOneWithEagerRelationships(id);
        return ResponseUtil.wrapOrNotFound(referenceTom);
    }

    /**
     * {@code DELETE  /reference-toms/:id} : delete the "id" referenceTom.
     *
     * @param id the id of the referenceTom to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteReferenceTom(@PathVariable("id") Long id) {
        log.debug("REST request to delete ReferenceTom : {}", id);
        referenceTomRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
