package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.ResourceType;
import com.emerald.rcm.backend.repository.ResourceTypeRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.ResourceType}.
 */
@RestController
@RequestMapping("/api/resource-types")
@Transactional
public class ResourceTypeResource {

    private static final Logger log = LoggerFactory.getLogger(ResourceTypeResource.class);

    private static final String ENTITY_NAME = "rcmBackendResourceType";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final ResourceTypeRepository resourceTypeRepository;

    public ResourceTypeResource(ResourceTypeRepository resourceTypeRepository) {
        this.resourceTypeRepository = resourceTypeRepository;
    }

    /**
     * {@code POST  /resource-types} : Create a new resourceType.
     *
     * @param resourceType the resourceType to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new resourceType, or with status {@code 400 (Bad Request)} if the resourceType has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<ResourceType> createResourceType(@Valid @RequestBody ResourceType resourceType) throws URISyntaxException {
        log.debug("REST request to save ResourceType : {}", resourceType);
        if (resourceType.getId() != null) {
            throw new BadRequestAlertException("A new resourceType cannot already have an ID", ENTITY_NAME, "idexists");
        }
        resourceType = resourceTypeRepository.save(resourceType);
        return ResponseEntity.created(new URI("/api/resource-types/" + resourceType.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, resourceType.getId().toString()))
            .body(resourceType);
    }

    /**
     * {@code PUT  /resource-types/:id} : Updates an existing resourceType.
     *
     * @param id the id of the resourceType to save.
     * @param resourceType the resourceType to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated resourceType,
     * or with status {@code 400 (Bad Request)} if the resourceType is not valid,
     * or with status {@code 500 (Internal Server Error)} if the resourceType couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<ResourceType> updateResourceType(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody ResourceType resourceType
    ) throws URISyntaxException {
        log.debug("REST request to update ResourceType : {}, {}", id, resourceType);
        if (resourceType.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, resourceType.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!resourceTypeRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        resourceType = resourceTypeRepository.save(resourceType);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, resourceType.getId().toString()))
            .body(resourceType);
    }

    /**
     * {@code PATCH  /resource-types/:id} : Partial updates given fields of an existing resourceType, field will ignore if it is null
     *
     * @param id the id of the resourceType to save.
     * @param resourceType the resourceType to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated resourceType,
     * or with status {@code 400 (Bad Request)} if the resourceType is not valid,
     * or with status {@code 404 (Not Found)} if the resourceType is not found,
     * or with status {@code 500 (Internal Server Error)} if the resourceType couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<ResourceType> partialUpdateResourceType(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody ResourceType resourceType
    ) throws URISyntaxException {
        log.debug("REST request to partial update ResourceType partially : {}, {}", id, resourceType);
        if (resourceType.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, resourceType.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!resourceTypeRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<ResourceType> result = resourceTypeRepository
            .findById(resourceType.getId())
            .map(existingResourceType -> {
                if (resourceType.getName() != null) {
                    existingResourceType.setName(resourceType.getName());
                }
                if (resourceType.getOntologyTerm() != null) {
                    existingResourceType.setOntologyTerm(resourceType.getOntologyTerm());
                }

                return existingResourceType;
            })
            .map(resourceTypeRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, resourceType.getId().toString())
        );
    }

    /**
     * {@code GET  /resource-types} : get all the resourceTypes.
     *
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of resourceTypes in body.
     */
    @GetMapping("")
    public List<ResourceType> getAllResourceTypes() {
        log.debug("REST request to get all ResourceTypes");
        return resourceTypeRepository.findAll();
    }

    /**
     * {@code GET  /resource-types/:id} : get the "id" resourceType.
     *
     * @param id the id of the resourceType to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the resourceType, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<ResourceType> getResourceType(@PathVariable("id") Long id) {
        log.debug("REST request to get ResourceType : {}", id);
        Optional<ResourceType> resourceType = resourceTypeRepository.findById(id);
        return ResponseUtil.wrapOrNotFound(resourceType);
    }

    /**
     * {@code DELETE  /resource-types/:id} : delete the "id" resourceType.
     *
     * @param id the id of the resourceType to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteResourceType(@PathVariable("id") Long id) {
        log.debug("REST request to delete ResourceType : {}", id);
        resourceTypeRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
