package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.SecurityControlCategory;
import com.emerald.rcm.backend.repository.SecurityControlCategoryRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.SecurityControlCategory}.
 */
@RestController
@RequestMapping("/api/security-control-categories")
@Transactional
public class SecurityControlCategoryResource {

    private static final Logger log = LoggerFactory.getLogger(SecurityControlCategoryResource.class);

    private static final String ENTITY_NAME = "rcmBackendSecurityControlCategory";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final SecurityControlCategoryRepository securityControlCategoryRepository;

    public SecurityControlCategoryResource(SecurityControlCategoryRepository securityControlCategoryRepository) {
        this.securityControlCategoryRepository = securityControlCategoryRepository;
    }

    /**
     * {@code POST  /security-control-categories} : Create a new securityControlCategory.
     *
     * @param securityControlCategory the securityControlCategory to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new securityControlCategory, or with status {@code 400 (Bad Request)} if the securityControlCategory has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<SecurityControlCategory> createSecurityControlCategory(
        @Valid @RequestBody SecurityControlCategory securityControlCategory
    ) throws URISyntaxException {
        log.debug("REST request to save SecurityControlCategory : {}", securityControlCategory);
        if (securityControlCategory.getId() != null) {
            throw new BadRequestAlertException("A new securityControlCategory cannot already have an ID", ENTITY_NAME, "idexists");
        }
        securityControlCategory = securityControlCategoryRepository.save(securityControlCategory);
        return ResponseEntity.created(new URI("/api/security-control-categories/" + securityControlCategory.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, securityControlCategory.getId().toString()))
            .body(securityControlCategory);
    }

    /**
     * {@code PUT  /security-control-categories/:id} : Updates an existing securityControlCategory.
     *
     * @param id the id of the securityControlCategory to save.
     * @param securityControlCategory the securityControlCategory to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated securityControlCategory,
     * or with status {@code 400 (Bad Request)} if the securityControlCategory is not valid,
     * or with status {@code 500 (Internal Server Error)} if the securityControlCategory couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<SecurityControlCategory> updateSecurityControlCategory(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody SecurityControlCategory securityControlCategory
    ) throws URISyntaxException {
        log.debug("REST request to update SecurityControlCategory : {}, {}", id, securityControlCategory);
        if (securityControlCategory.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, securityControlCategory.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!securityControlCategoryRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        securityControlCategory = securityControlCategoryRepository.save(securityControlCategory);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, securityControlCategory.getId().toString()))
            .body(securityControlCategory);
    }

    /**
     * {@code PATCH  /security-control-categories/:id} : Partial updates given fields of an existing securityControlCategory, field will ignore if it is null
     *
     * @param id the id of the securityControlCategory to save.
     * @param securityControlCategory the securityControlCategory to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated securityControlCategory,
     * or with status {@code 400 (Bad Request)} if the securityControlCategory is not valid,
     * or with status {@code 404 (Not Found)} if the securityControlCategory is not found,
     * or with status {@code 500 (Internal Server Error)} if the securityControlCategory couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<SecurityControlCategory> partialUpdateSecurityControlCategory(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody SecurityControlCategory securityControlCategory
    ) throws URISyntaxException {
        log.debug("REST request to partial update SecurityControlCategory partially : {}, {}", id, securityControlCategory);
        if (securityControlCategory.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, securityControlCategory.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!securityControlCategoryRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<SecurityControlCategory> result = securityControlCategoryRepository
            .findById(securityControlCategory.getId())
            .map(existingSecurityControlCategory -> {
                if (securityControlCategory.getCode() != null) {
                    existingSecurityControlCategory.setCode(securityControlCategory.getCode());
                }
                if (securityControlCategory.getName() != null) {
                    existingSecurityControlCategory.setName(securityControlCategory.getName());
                }
                if (securityControlCategory.getDescription() != null) {
                    existingSecurityControlCategory.setDescription(securityControlCategory.getDescription());
                }

                return existingSecurityControlCategory;
            })
            .map(securityControlCategoryRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, securityControlCategory.getId().toString())
        );
    }

    /**
     * {@code GET  /security-control-categories} : get all the securityControlCategories.
     *
     * @param eagerload flag to eager load entities from relationships (This is applicable for many-to-many).
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of securityControlCategories in body.
     */
    @GetMapping("")
    public List<SecurityControlCategory> getAllSecurityControlCategories(
        @RequestParam(name = "eagerload", required = false, defaultValue = "true") boolean eagerload
    ) {
        log.debug("REST request to get all SecurityControlCategories");
        if (eagerload) {
            return securityControlCategoryRepository.findAllWithEagerRelationships();
        } else {
            return securityControlCategoryRepository.findAll();
        }
    }

    /**
     * {@code GET  /security-control-categories/:id} : get the "id" securityControlCategory.
     *
     * @param id the id of the securityControlCategory to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the securityControlCategory, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<SecurityControlCategory> getSecurityControlCategory(@PathVariable("id") Long id) {
        log.debug("REST request to get SecurityControlCategory : {}", id);
        Optional<SecurityControlCategory> securityControlCategory = securityControlCategoryRepository.findOneWithEagerRelationships(id);
        return ResponseUtil.wrapOrNotFound(securityControlCategory);
    }

    /**
     * {@code DELETE  /security-control-categories/:id} : delete the "id" securityControlCategory.
     *
     * @param id the id of the securityControlCategory to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteSecurityControlCategory(@PathVariable("id") Long id) {
        log.debug("REST request to delete SecurityControlCategory : {}", id);
        securityControlCategoryRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
