package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.SecurityControlFramework;
import com.emerald.rcm.backend.repository.SecurityControlFrameworkRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.SecurityControlFramework}.
 */
@RestController
@RequestMapping("/api/security-control-frameworks")
@Transactional
public class SecurityControlFrameworkResource {

    private static final Logger log = LoggerFactory.getLogger(SecurityControlFrameworkResource.class);

    private static final String ENTITY_NAME = "rcmBackendSecurityControlFramework";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final SecurityControlFrameworkRepository securityControlFrameworkRepository;

    public SecurityControlFrameworkResource(SecurityControlFrameworkRepository securityControlFrameworkRepository) {
        this.securityControlFrameworkRepository = securityControlFrameworkRepository;
    }

    /**
     * {@code POST  /security-control-frameworks} : Create a new securityControlFramework.
     *
     * @param securityControlFramework the securityControlFramework to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new securityControlFramework, or with status {@code 400 (Bad Request)} if the securityControlFramework has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<SecurityControlFramework> createSecurityControlFramework(
        @Valid @RequestBody SecurityControlFramework securityControlFramework
    ) throws URISyntaxException {
        log.debug("REST request to save SecurityControlFramework : {}", securityControlFramework);
        if (securityControlFramework.getId() != null) {
            throw new BadRequestAlertException("A new securityControlFramework cannot already have an ID", ENTITY_NAME, "idexists");
        }
        securityControlFramework = securityControlFrameworkRepository.save(securityControlFramework);
        return ResponseEntity.created(new URI("/api/security-control-frameworks/" + securityControlFramework.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, securityControlFramework.getId().toString()))
            .body(securityControlFramework);
    }

    /**
     * {@code PUT  /security-control-frameworks/:id} : Updates an existing securityControlFramework.
     *
     * @param id the id of the securityControlFramework to save.
     * @param securityControlFramework the securityControlFramework to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated securityControlFramework,
     * or with status {@code 400 (Bad Request)} if the securityControlFramework is not valid,
     * or with status {@code 500 (Internal Server Error)} if the securityControlFramework couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<SecurityControlFramework> updateSecurityControlFramework(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody SecurityControlFramework securityControlFramework
    ) throws URISyntaxException {
        log.debug("REST request to update SecurityControlFramework : {}, {}", id, securityControlFramework);
        if (securityControlFramework.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, securityControlFramework.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!securityControlFrameworkRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        securityControlFramework = securityControlFrameworkRepository.save(securityControlFramework);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, securityControlFramework.getId().toString()))
            .body(securityControlFramework);
    }

    /**
     * {@code PATCH  /security-control-frameworks/:id} : Partial updates given fields of an existing securityControlFramework, field will ignore if it is null
     *
     * @param id the id of the securityControlFramework to save.
     * @param securityControlFramework the securityControlFramework to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated securityControlFramework,
     * or with status {@code 400 (Bad Request)} if the securityControlFramework is not valid,
     * or with status {@code 404 (Not Found)} if the securityControlFramework is not found,
     * or with status {@code 500 (Internal Server Error)} if the securityControlFramework couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<SecurityControlFramework> partialUpdateSecurityControlFramework(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody SecurityControlFramework securityControlFramework
    ) throws URISyntaxException {
        log.debug("REST request to partial update SecurityControlFramework partially : {}, {}", id, securityControlFramework);
        if (securityControlFramework.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, securityControlFramework.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!securityControlFrameworkRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<SecurityControlFramework> result = securityControlFrameworkRepository
            .findById(securityControlFramework.getId())
            .map(existingSecurityControlFramework -> {
                if (securityControlFramework.getName() != null) {
                    existingSecurityControlFramework.setName(securityControlFramework.getName());
                }
                if (securityControlFramework.getDescription() != null) {
                    existingSecurityControlFramework.setDescription(securityControlFramework.getDescription());
                }
                if (securityControlFramework.getVersion() != null) {
                    existingSecurityControlFramework.setVersion(securityControlFramework.getVersion());
                }

                return existingSecurityControlFramework;
            })
            .map(securityControlFrameworkRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, securityControlFramework.getId().toString())
        );
    }

    /**
     * {@code GET  /security-control-frameworks} : get all the securityControlFrameworks.
     *
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of securityControlFrameworks in body.
     */
    @GetMapping("")
    public List<SecurityControlFramework> getAllSecurityControlFrameworks() {
        log.debug("REST request to get all SecurityControlFrameworks");
        return securityControlFrameworkRepository.findAll();
    }

    /**
     * {@code GET  /security-control-frameworks/:id} : get the "id" securityControlFramework.
     *
     * @param id the id of the securityControlFramework to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the securityControlFramework, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<SecurityControlFramework> getSecurityControlFramework(@PathVariable("id") Long id) {
        log.debug("REST request to get SecurityControlFramework : {}", id);
        Optional<SecurityControlFramework> securityControlFramework = securityControlFrameworkRepository.findById(id);
        return ResponseUtil.wrapOrNotFound(securityControlFramework);
    }

    /**
     * {@code DELETE  /security-control-frameworks/:id} : delete the "id" securityControlFramework.
     *
     * @param id the id of the securityControlFramework to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteSecurityControlFramework(@PathVariable("id") Long id) {
        log.debug("REST request to delete SecurityControlFramework : {}", id);
        securityControlFrameworkRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
