package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.SecurityControl;
import com.emerald.rcm.backend.repository.SecurityControlRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.SecurityControl}.
 */
@RestController
@RequestMapping("/api/security-controls")
@Transactional
public class SecurityControlResource {

    private static final Logger log = LoggerFactory.getLogger(SecurityControlResource.class);

    private static final String ENTITY_NAME = "rcmBackendSecurityControl";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final SecurityControlRepository securityControlRepository;

    public SecurityControlResource(SecurityControlRepository securityControlRepository) {
        this.securityControlRepository = securityControlRepository;
    }

    /**
     * {@code POST  /security-controls} : Create a new securityControl.
     *
     * @param securityControl the securityControl to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new securityControl, or with status {@code 400 (Bad Request)} if the securityControl has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<SecurityControl> createSecurityControl(@Valid @RequestBody SecurityControl securityControl)
        throws URISyntaxException {
        log.debug("REST request to save SecurityControl : {}", securityControl);
        if (securityControl.getId() != null) {
            throw new BadRequestAlertException("A new securityControl cannot already have an ID", ENTITY_NAME, "idexists");
        }
        securityControl = securityControlRepository.save(securityControl);
        return ResponseEntity.created(new URI("/api/security-controls/" + securityControl.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, securityControl.getId().toString()))
            .body(securityControl);
    }

    /**
     * {@code PUT  /security-controls/:id} : Updates an existing securityControl.
     *
     * @param id the id of the securityControl to save.
     * @param securityControl the securityControl to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated securityControl,
     * or with status {@code 400 (Bad Request)} if the securityControl is not valid,
     * or with status {@code 500 (Internal Server Error)} if the securityControl couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<SecurityControl> updateSecurityControl(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody SecurityControl securityControl
    ) throws URISyntaxException {
        log.debug("REST request to update SecurityControl : {}, {}", id, securityControl);
        if (securityControl.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, securityControl.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!securityControlRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        securityControl = securityControlRepository.save(securityControl);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, securityControl.getId().toString()))
            .body(securityControl);
    }

    /**
     * {@code PATCH  /security-controls/:id} : Partial updates given fields of an existing securityControl, field will ignore if it is null
     *
     * @param id the id of the securityControl to save.
     * @param securityControl the securityControl to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated securityControl,
     * or with status {@code 400 (Bad Request)} if the securityControl is not valid,
     * or with status {@code 404 (Not Found)} if the securityControl is not found,
     * or with status {@code 500 (Internal Server Error)} if the securityControl couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<SecurityControl> partialUpdateSecurityControl(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody SecurityControl securityControl
    ) throws URISyntaxException {
        log.debug("REST request to partial update SecurityControl partially : {}, {}", id, securityControl);
        if (securityControl.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, securityControl.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!securityControlRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<SecurityControl> result = securityControlRepository
            .findById(securityControl.getId())
            .map(existingSecurityControl -> {
                if (securityControl.getCode() != null) {
                    existingSecurityControl.setCode(securityControl.getCode());
                }
                if (securityControl.getName() != null) {
                    existingSecurityControl.setName(securityControl.getName());
                }
                if (securityControl.getObjective() != null) {
                    existingSecurityControl.setObjective(securityControl.getObjective());
                }
                if (securityControl.getDescription() != null) {
                    existingSecurityControl.setDescription(securityControl.getDescription());
                }

                return existingSecurityControl;
            })
            .map(securityControlRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, securityControl.getId().toString())
        );
    }

    /**
     * {@code GET  /security-controls} : get all the securityControls.
     *
     * @param eagerload flag to eager load entities from relationships (This is applicable for many-to-many).
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of securityControls in body.
     */
    @GetMapping("")
    public List<SecurityControl> getAllSecurityControls(
        @RequestParam(name = "eagerload", required = false, defaultValue = "true") boolean eagerload
    ) {
        log.debug("REST request to get all SecurityControls");
        if (eagerload) {
            return securityControlRepository.findAllWithEagerRelationships();
        } else {
            return securityControlRepository.findAll();
        }
    }

    /**
     * {@code GET  /security-controls/:id} : get the "id" securityControl.
     *
     * @param id the id of the securityControl to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the securityControl, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<SecurityControl> getSecurityControl(@PathVariable("id") Long id) {
        log.debug("REST request to get SecurityControl : {}", id);
        Optional<SecurityControl> securityControl = securityControlRepository.findOneWithEagerRelationships(id);
        return ResponseUtil.wrapOrNotFound(securityControl);
    }

    /**
     * {@code DELETE  /security-controls/:id} : delete the "id" securityControl.
     *
     * @param id the id of the securityControl to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteSecurityControl(@PathVariable("id") Long id) {
        log.debug("REST request to delete SecurityControl : {}", id);
        securityControlRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
