package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.SecurityMetric;
import com.emerald.rcm.backend.repository.SecurityMetricRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.SecurityMetric}.
 */
@RestController
@RequestMapping("/api/security-metrics")
@Transactional
public class SecurityMetricResource {

    private static final Logger log = LoggerFactory.getLogger(SecurityMetricResource.class);

    private static final String ENTITY_NAME = "rcmBackendSecurityMetric";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final SecurityMetricRepository securityMetricRepository;

    public SecurityMetricResource(SecurityMetricRepository securityMetricRepository) {
        this.securityMetricRepository = securityMetricRepository;
    }

    /**
     * {@code POST  /security-metrics} : Create a new securityMetric.
     *
     * @param securityMetric the securityMetric to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new securityMetric, or with status {@code 400 (Bad Request)} if the securityMetric has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<SecurityMetric> createSecurityMetric(@Valid @RequestBody SecurityMetric securityMetric)
        throws URISyntaxException {
        log.debug("REST request to save SecurityMetric : {}", securityMetric);
        if (securityMetric.getId() != null) {
            throw new BadRequestAlertException("A new securityMetric cannot already have an ID", ENTITY_NAME, "idexists");
        }
        securityMetric = securityMetricRepository.save(securityMetric);
        return ResponseEntity.created(new URI("/api/security-metrics/" + securityMetric.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, securityMetric.getId().toString()))
            .body(securityMetric);
    }

    /**
     * {@code PUT  /security-metrics/:id} : Updates an existing securityMetric.
     *
     * @param id the id of the securityMetric to save.
     * @param securityMetric the securityMetric to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated securityMetric,
     * or with status {@code 400 (Bad Request)} if the securityMetric is not valid,
     * or with status {@code 500 (Internal Server Error)} if the securityMetric couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<SecurityMetric> updateSecurityMetric(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody SecurityMetric securityMetric
    ) throws URISyntaxException {
        log.debug("REST request to update SecurityMetric : {}, {}", id, securityMetric);
        if (securityMetric.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, securityMetric.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!securityMetricRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        securityMetric = securityMetricRepository.save(securityMetric);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, securityMetric.getId().toString()))
            .body(securityMetric);
    }

    /**
     * {@code PATCH  /security-metrics/:id} : Partial updates given fields of an existing securityMetric, field will ignore if it is null
     *
     * @param id the id of the securityMetric to save.
     * @param securityMetric the securityMetric to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated securityMetric,
     * or with status {@code 400 (Bad Request)} if the securityMetric is not valid,
     * or with status {@code 404 (Not Found)} if the securityMetric is not found,
     * or with status {@code 500 (Internal Server Error)} if the securityMetric couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<SecurityMetric> partialUpdateSecurityMetric(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody SecurityMetric securityMetric
    ) throws URISyntaxException {
        log.debug("REST request to partial update SecurityMetric partially : {}, {}", id, securityMetric);
        if (securityMetric.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, securityMetric.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!securityMetricRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<SecurityMetric> result = securityMetricRepository
            .findById(securityMetric.getId())
            .map(existingSecurityMetric -> {
                if (securityMetric.getMetricId() != null) {
                    existingSecurityMetric.setMetricId(securityMetric.getMetricId());
                }
                if (securityMetric.getCategory() != null) {
                    existingSecurityMetric.setCategory(securityMetric.getCategory());
                }
                if (securityMetric.getName() != null) {
                    existingSecurityMetric.setName(securityMetric.getName());
                }
                if (securityMetric.getSource() != null) {
                    existingSecurityMetric.setSource(securityMetric.getSource());
                }
                if (securityMetric.getDescription() != null) {
                    existingSecurityMetric.setDescription(securityMetric.getDescription());
                }
                if (securityMetric.getScale() != null) {
                    existingSecurityMetric.setScale(securityMetric.getScale());
                }
                if (securityMetric.getOperator() != null) {
                    existingSecurityMetric.setOperator(securityMetric.getOperator());
                }
                if (securityMetric.getTargetValue() != null) {
                    existingSecurityMetric.setTargetValue(securityMetric.getTargetValue());
                }
                if (securityMetric.getTargetValueDatatype() != null) {
                    existingSecurityMetric.setTargetValueDatatype(securityMetric.getTargetValueDatatype());
                }
                if (securityMetric.getInterval() != null) {
                    existingSecurityMetric.setInterval(securityMetric.getInterval());
                }
                if (securityMetric.getTargetResourceType() != null) {
                    existingSecurityMetric.setTargetResourceType(securityMetric.getTargetResourceType());
                }
                if (securityMetric.getSecurityFeature() != null) {
                    existingSecurityMetric.setSecurityFeature(securityMetric.getSecurityFeature());
                }
                if (securityMetric.getKeywords() != null) {
                    existingSecurityMetric.setKeywords(securityMetric.getKeywords());
                }

                return existingSecurityMetric;
            })
            .map(securityMetricRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, securityMetric.getId().toString())
        );
    }

    /**
     * {@code GET  /security-metrics} : get all the securityMetrics.
     *
     * @param eagerload flag to eager load entities from relationships (This is applicable for many-to-many).
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of securityMetrics in body.
     */
    @GetMapping("")
    public List<SecurityMetric> getAllSecurityMetrics(
        @RequestParam(name = "eagerload", required = false, defaultValue = "true") boolean eagerload
    ) {
        log.debug("REST request to get all SecurityMetrics");
        if (eagerload) {
            return securityMetricRepository.findAllWithEagerRelationships();
        } else {
            return securityMetricRepository.findAll();
        }
    }

    /**
     * {@code GET  /security-metrics/:id} : get the "id" securityMetric.
     *
     * @param id the id of the securityMetric to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the securityMetric, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<SecurityMetric> getSecurityMetric(@PathVariable("id") Long id) {
        log.debug("REST request to get SecurityMetric : {}", id);
        Optional<SecurityMetric> securityMetric = securityMetricRepository.findOneWithEagerRelationships(id);
        return ResponseUtil.wrapOrNotFound(securityMetric);
    }

    /**
     * {@code DELETE  /security-metrics/:id} : delete the "id" securityMetric.
     *
     * @param id the id of the securityMetric to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteSecurityMetric(@PathVariable("id") Long id) {
        log.debug("REST request to delete SecurityMetric : {}", id);
        securityMetricRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
