package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.SimilarControl;
import com.emerald.rcm.backend.repository.SimilarControlRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.SimilarControl}.
 */
@RestController
@RequestMapping("/api/similar-controls")
@Transactional
public class SimilarControlResource {

    private static final Logger log = LoggerFactory.getLogger(SimilarControlResource.class);

    private static final String ENTITY_NAME = "rcmBackendSimilarControl";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final SimilarControlRepository similarControlRepository;

    public SimilarControlResource(SimilarControlRepository similarControlRepository) {
        this.similarControlRepository = similarControlRepository;
    }

    /**
     * {@code POST  /similar-controls} : Create a new similarControl.
     *
     * @param similarControl the similarControl to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new similarControl, or with status {@code 400 (Bad Request)} if the similarControl has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<SimilarControl> createSimilarControl(@Valid @RequestBody SimilarControl similarControl)
        throws URISyntaxException {
        log.debug("REST request to save SimilarControl : {}", similarControl);
        if (similarControl.getId() != null) {
            throw new BadRequestAlertException("A new similarControl cannot already have an ID", ENTITY_NAME, "idexists");
        }
        similarControl = similarControlRepository.save(similarControl);
        return ResponseEntity.created(new URI("/api/similar-controls/" + similarControl.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, similarControl.getId().toString()))
            .body(similarControl);
    }

    /**
     * {@code PUT  /similar-controls/:id} : Updates an existing similarControl.
     *
     * @param id the id of the similarControl to save.
     * @param similarControl the similarControl to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated similarControl,
     * or with status {@code 400 (Bad Request)} if the similarControl is not valid,
     * or with status {@code 500 (Internal Server Error)} if the similarControl couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<SimilarControl> updateSimilarControl(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody SimilarControl similarControl
    ) throws URISyntaxException {
        log.debug("REST request to update SimilarControl : {}, {}", id, similarControl);
        if (similarControl.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, similarControl.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!similarControlRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        similarControl = similarControlRepository.save(similarControl);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, similarControl.getId().toString()))
            .body(similarControl);
    }

    /**
     * {@code PATCH  /similar-controls/:id} : Partial updates given fields of an existing similarControl, field will ignore if it is null
     *
     * @param id the id of the similarControl to save.
     * @param similarControl the similarControl to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated similarControl,
     * or with status {@code 400 (Bad Request)} if the similarControl is not valid,
     * or with status {@code 404 (Not Found)} if the similarControl is not found,
     * or with status {@code 500 (Internal Server Error)} if the similarControl couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<SimilarControl> partialUpdateSimilarControl(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody SimilarControl similarControl
    ) throws URISyntaxException {
        log.debug("REST request to partial update SimilarControl partially : {}, {}", id, similarControl);
        if (similarControl.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, similarControl.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!similarControlRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<SimilarControl> result = similarControlRepository
            .findById(similarControl.getId())
            .map(existingSimilarControl -> {
                if (similarControl.getCode() != null) {
                    existingSimilarControl.setCode(similarControl.getCode());
                }
                if (similarControl.getName() != null) {
                    existingSimilarControl.setName(similarControl.getName());
                }
                if (similarControl.getSecurityControlFrameworkName() != null) {
                    existingSimilarControl.setSecurityControlFrameworkName(similarControl.getSecurityControlFrameworkName());
                }

                return existingSimilarControl;
            })
            .map(similarControlRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, similarControl.getId().toString())
        );
    }

    /**
     * {@code GET  /similar-controls} : get all the similarControls.
     *
     * @param eagerload flag to eager load entities from relationships (This is applicable for many-to-many).
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of similarControls in body.
     */
    @GetMapping("")
    public List<SimilarControl> getAllSimilarControls(
        @RequestParam(name = "eagerload", required = false, defaultValue = "true") boolean eagerload
    ) {
        log.debug("REST request to get all SimilarControls");
        if (eagerload) {
            return similarControlRepository.findAllWithEagerRelationships();
        } else {
            return similarControlRepository.findAll();
        }
    }

    /**
     * {@code GET  /similar-controls/:id} : get the "id" similarControl.
     *
     * @param id the id of the similarControl to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the similarControl, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<SimilarControl> getSimilarControl(@PathVariable("id") Long id) {
        log.debug("REST request to get SimilarControl : {}", id);
        Optional<SimilarControl> similarControl = similarControlRepository.findOneWithEagerRelationships(id);
        return ResponseUtil.wrapOrNotFound(similarControl);
    }

    /**
     * {@code DELETE  /similar-controls/:id} : delete the "id" similarControl.
     *
     * @param id the id of the similarControl to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteSimilarControl(@PathVariable("id") Long id) {
        log.debug("REST request to delete SimilarControl : {}", id);
        similarControlRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
