package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.TargetValue;
import com.emerald.rcm.backend.repository.TargetValueRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.TargetValue}.
 */
@RestController
@RequestMapping("/api/target-values")
@Transactional
public class TargetValueResource {

    private static final Logger log = LoggerFactory.getLogger(TargetValueResource.class);

    private static final String ENTITY_NAME = "rcmBackendTargetValue";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final TargetValueRepository targetValueRepository;

    public TargetValueResource(TargetValueRepository targetValueRepository) {
        this.targetValueRepository = targetValueRepository;
    }

    /**
     * {@code POST  /target-values} : Create a new targetValue.
     *
     * @param targetValue the targetValue to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new targetValue, or with status {@code 400 (Bad Request)} if the targetValue has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<TargetValue> createTargetValue(@Valid @RequestBody TargetValue targetValue) throws URISyntaxException {
        log.debug("REST request to save TargetValue : {}", targetValue);
        if (targetValue.getId() != null) {
            throw new BadRequestAlertException("A new targetValue cannot already have an ID", ENTITY_NAME, "idexists");
        }
        targetValue = targetValueRepository.save(targetValue);
        return ResponseEntity.created(new URI("/api/target-values/" + targetValue.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, targetValue.getId().toString()))
            .body(targetValue);
    }

    /**
     * {@code PUT  /target-values/:id} : Updates an existing targetValue.
     *
     * @param id the id of the targetValue to save.
     * @param targetValue the targetValue to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated targetValue,
     * or with status {@code 400 (Bad Request)} if the targetValue is not valid,
     * or with status {@code 500 (Internal Server Error)} if the targetValue couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<TargetValue> updateTargetValue(
        @PathVariable(value = "id", required = false) final Long id,
        @Valid @RequestBody TargetValue targetValue
    ) throws URISyntaxException {
        log.debug("REST request to update TargetValue : {}, {}", id, targetValue);
        if (targetValue.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, targetValue.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!targetValueRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        targetValue = targetValueRepository.save(targetValue);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, targetValue.getId().toString()))
            .body(targetValue);
    }

    /**
     * {@code PATCH  /target-values/:id} : Partial updates given fields of an existing targetValue, field will ignore if it is null
     *
     * @param id the id of the targetValue to save.
     * @param targetValue the targetValue to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated targetValue,
     * or with status {@code 400 (Bad Request)} if the targetValue is not valid,
     * or with status {@code 404 (Not Found)} if the targetValue is not found,
     * or with status {@code 500 (Internal Server Error)} if the targetValue couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<TargetValue> partialUpdateTargetValue(
        @PathVariable(value = "id", required = false) final Long id,
        @NotNull @RequestBody TargetValue targetValue
    ) throws URISyntaxException {
        log.debug("REST request to partial update TargetValue partially : {}, {}", id, targetValue);
        if (targetValue.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, targetValue.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!targetValueRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<TargetValue> result = targetValueRepository
            .findById(targetValue.getId())
            .map(existingTargetValue -> {
                if (targetValue.getValue() != null) {
                    existingTargetValue.setValue(targetValue.getValue());
                }

                return existingTargetValue;
            })
            .map(targetValueRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, targetValue.getId().toString())
        );
    }

    /**
     * {@code GET  /target-values} : get all the targetValues.
     *
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of targetValues in body.
     */
    @GetMapping("")
    public List<TargetValue> getAllTargetValues() {
        log.debug("REST request to get all TargetValues");
        return targetValueRepository.findAll();
    }

    /**
     * {@code GET  /target-values/:id} : get the "id" targetValue.
     *
     * @param id the id of the targetValue to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the targetValue, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<TargetValue> getTargetValue(@PathVariable("id") Long id) {
        log.debug("REST request to get TargetValue : {}", id);
        Optional<TargetValue> targetValue = targetValueRepository.findById(id);
        return ResponseUtil.wrapOrNotFound(targetValue);
    }

    /**
     * {@code DELETE  /target-values/:id} : delete the "id" targetValue.
     *
     * @param id the id of the targetValue to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteTargetValue(@PathVariable("id") Long id) {
        log.debug("REST request to delete TargetValue : {}", id);
        targetValueRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
