package com.emerald.rcm.backend.web.rest;

import com.emerald.rcm.backend.domain.Tom;
import com.emerald.rcm.backend.repository.TomRepository;
import com.emerald.rcm.backend.web.rest.errors.BadRequestAlertException;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import tech.jhipster.web.util.HeaderUtil;
import tech.jhipster.web.util.ResponseUtil;

/**
 * REST controller for managing {@link com.emerald.rcm.backend.domain.Tom}.
 */
@RestController
@RequestMapping("/api/toms")
@Transactional
public class TomResource {

    private static final Logger log = LoggerFactory.getLogger(TomResource.class);

    private static final String ENTITY_NAME = "rcmBackendTom";

    @Value("${jhipster.clientApp.name}")
    private String applicationName;

    private final TomRepository tomRepository;

    public TomResource(TomRepository tomRepository) {
        this.tomRepository = tomRepository;
    }

    /**
     * {@code POST  /toms} : Create a new tom.
     *
     * @param tom the tom to create.
     * @return the {@link ResponseEntity} with status {@code 201 (Created)} and with body the new tom, or with status {@code 400 (Bad Request)} if the tom has already an ID.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PostMapping("")
    public ResponseEntity<Tom> createTom(@Valid @RequestBody Tom tom) throws URISyntaxException {
        log.debug("REST request to save Tom : {}", tom);
        if (tom.getId() != null) {
            throw new BadRequestAlertException("A new tom cannot already have an ID", ENTITY_NAME, "idexists");
        }
        tom = tomRepository.save(tom);
        return ResponseEntity.created(new URI("/api/toms/" + tom.getId()))
            .headers(HeaderUtil.createEntityCreationAlert(applicationName, true, ENTITY_NAME, tom.getId().toString()))
            .body(tom);
    }

    /**
     * {@code PUT  /toms/:id} : Updates an existing tom.
     *
     * @param id the id of the tom to save.
     * @param tom the tom to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated tom,
     * or with status {@code 400 (Bad Request)} if the tom is not valid,
     * or with status {@code 500 (Internal Server Error)} if the tom couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PutMapping("/{id}")
    public ResponseEntity<Tom> updateTom(@PathVariable(value = "id", required = false) final Long id, @Valid @RequestBody Tom tom)
        throws URISyntaxException {
        log.debug("REST request to update Tom : {}, {}", id, tom);
        if (tom.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, tom.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!tomRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        tom = tomRepository.save(tom);
        return ResponseEntity.ok()
            .headers(HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, tom.getId().toString()))
            .body(tom);
    }

    /**
     * {@code PATCH  /toms/:id} : Partial updates given fields of an existing tom, field will ignore if it is null
     *
     * @param id the id of the tom to save.
     * @param tom the tom to update.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the updated tom,
     * or with status {@code 400 (Bad Request)} if the tom is not valid,
     * or with status {@code 404 (Not Found)} if the tom is not found,
     * or with status {@code 500 (Internal Server Error)} if the tom couldn't be updated.
     * @throws URISyntaxException if the Location URI syntax is incorrect.
     */
    @PatchMapping(value = "/{id}", consumes = { "application/json", "application/merge-patch+json" })
    public ResponseEntity<Tom> partialUpdateTom(@PathVariable(value = "id", required = false) final Long id, @NotNull @RequestBody Tom tom)
        throws URISyntaxException {
        log.debug("REST request to partial update Tom partially : {}, {}", id, tom);
        if (tom.getId() == null) {
            throw new BadRequestAlertException("Invalid id", ENTITY_NAME, "idnull");
        }
        if (!Objects.equals(id, tom.getId())) {
            throw new BadRequestAlertException("Invalid ID", ENTITY_NAME, "idinvalid");
        }

        if (!tomRepository.existsById(id)) {
            throw new BadRequestAlertException("Entity not found", ENTITY_NAME, "idnotfound");
        }

        Optional<Tom> result = tomRepository
            .findById(tom.getId())
            .map(existingTom -> {
                if (tom.getCode() != null) {
                    existingTom.setCode(tom.getCode());
                }
                if (tom.getName() != null) {
                    existingTom.setName(tom.getName());
                }
                if (tom.getDescription() != null) {
                    existingTom.setDescription(tom.getDescription());
                }
                if (tom.getAssuranceLevel() != null) {
                    existingTom.setAssuranceLevel(tom.getAssuranceLevel());
                }
                if (tom.getType() != null) {
                    existingTom.setType(tom.getType());
                }
                if (tom.getGuidance() != null) {
                    existingTom.setGuidance(tom.getGuidance());
                }

                return existingTom;
            })
            .map(tomRepository::save);

        return ResponseUtil.wrapOrNotFound(
            result,
            HeaderUtil.createEntityUpdateAlert(applicationName, true, ENTITY_NAME, tom.getId().toString())
        );
    }

    /**
     * {@code GET  /toms} : get all the toms.
     *
     * @param eagerload flag to eager load entities from relationships (This is applicable for many-to-many).
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and the list of toms in body.
     */
    @GetMapping("")
    public List<Tom> getAllToms(@RequestParam(name = "eagerload", required = false, defaultValue = "true") boolean eagerload) {
        log.debug("REST request to get all Toms");
        if (eagerload) {
            return tomRepository.findAllWithEagerRelationships();
        } else {
            return tomRepository.findAll();
        }
    }

    /**
     * {@code GET  /toms/:id} : get the "id" tom.
     *
     * @param id the id of the tom to retrieve.
     * @return the {@link ResponseEntity} with status {@code 200 (OK)} and with body the tom, or with status {@code 404 (Not Found)}.
     */
    @GetMapping("/{id}")
    public ResponseEntity<Tom> getTom(@PathVariable("id") Long id) {
        log.debug("REST request to get Tom : {}", id);
        Optional<Tom> tom = tomRepository.findOneWithEagerRelationships(id);
        return ResponseUtil.wrapOrNotFound(tom);
    }

    /**
     * {@code DELETE  /toms/:id} : delete the "id" tom.
     *
     * @param id the id of the tom to delete.
     * @return the {@link ResponseEntity} with status {@code 204 (NO_CONTENT)}.
     */
    @DeleteMapping("/{id}")
    public ResponseEntity<Void> deleteTom(@PathVariable("id") Long id) {
        log.debug("REST request to delete Tom : {}", id);
        tomRepository.deleteById(id);
        return ResponseEntity.noContent()
            .headers(HeaderUtil.createEntityDeletionAlert(applicationName, true, ENTITY_NAME, id.toString()))
            .build();
    }
}
