package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.QuestionAnswerAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.QuestionAnswer;
import com.emerald.rcm.backend.repository.QuestionAnswerRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link QuestionAnswerResource} REST controller.
 */
@IntegrationTest
@AutoConfigureMockMvc
@WithMockUser
class QuestionAnswerResourceIT {

    private static final String DEFAULT_ANSWER = "AAAAAAAAAA";
    private static final String UPDATED_ANSWER = "BBBBBBBBBB";

    private static final Integer DEFAULT_VALUE = 1;
    private static final Integer UPDATED_VALUE = 2;

    private static final Boolean DEFAULT_VISIBLE = false;
    private static final Boolean UPDATED_VISIBLE = true;

    private static final String ENTITY_API_URL = "/api/question-answers";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private QuestionAnswerRepository questionAnswerRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restQuestionAnswerMockMvc;

    private QuestionAnswer questionAnswer;

    private QuestionAnswer insertedQuestionAnswer;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static QuestionAnswer createEntity(EntityManager em) {
        QuestionAnswer questionAnswer = new QuestionAnswer().answer(DEFAULT_ANSWER).value(DEFAULT_VALUE).visible(DEFAULT_VISIBLE);
        return questionAnswer;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static QuestionAnswer createUpdatedEntity(EntityManager em) {
        QuestionAnswer questionAnswer = new QuestionAnswer().answer(UPDATED_ANSWER).value(UPDATED_VALUE).visible(UPDATED_VISIBLE);
        return questionAnswer;
    }

    @BeforeEach
    public void initTest() {
        questionAnswer = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedQuestionAnswer != null) {
            questionAnswerRepository.delete(insertedQuestionAnswer);
            insertedQuestionAnswer = null;
        }
    }

    @Test
    @Transactional
    void createQuestionAnswer() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the QuestionAnswer
        var returnedQuestionAnswer = om.readValue(
            restQuestionAnswerMockMvc
                .perform(
                    post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionAnswer))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            QuestionAnswer.class
        );

        // Validate the QuestionAnswer in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertQuestionAnswerUpdatableFieldsEquals(returnedQuestionAnswer, getPersistedQuestionAnswer(returnedQuestionAnswer));

        insertedQuestionAnswer = returnedQuestionAnswer;
    }

    @Test
    @Transactional
    void createQuestionAnswerWithExistingId() throws Exception {
        // Create the QuestionAnswer with an existing ID
        questionAnswer.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restQuestionAnswerMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionAnswer))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionAnswer in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkAnswerIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        questionAnswer.setAnswer(null);

        // Create the QuestionAnswer, which fails.

        restQuestionAnswerMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionAnswer))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkValueIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        questionAnswer.setValue(null);

        // Create the QuestionAnswer, which fails.

        restQuestionAnswerMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionAnswer))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkVisibleIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        questionAnswer.setVisible(null);

        // Create the QuestionAnswer, which fails.

        restQuestionAnswerMockMvc
            .perform(
                post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionAnswer))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllQuestionAnswers() throws Exception {
        // Initialize the database
        insertedQuestionAnswer = questionAnswerRepository.saveAndFlush(questionAnswer);

        // Get all the questionAnswerList
        restQuestionAnswerMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(questionAnswer.getId().intValue())))
            .andExpect(jsonPath("$.[*].answer").value(hasItem(DEFAULT_ANSWER)))
            .andExpect(jsonPath("$.[*].value").value(hasItem(DEFAULT_VALUE)))
            .andExpect(jsonPath("$.[*].visible").value(hasItem(DEFAULT_VISIBLE.booleanValue())));
    }

    @Test
    @Transactional
    void getQuestionAnswer() throws Exception {
        // Initialize the database
        insertedQuestionAnswer = questionAnswerRepository.saveAndFlush(questionAnswer);

        // Get the questionAnswer
        restQuestionAnswerMockMvc
            .perform(get(ENTITY_API_URL_ID, questionAnswer.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(questionAnswer.getId().intValue()))
            .andExpect(jsonPath("$.answer").value(DEFAULT_ANSWER))
            .andExpect(jsonPath("$.value").value(DEFAULT_VALUE))
            .andExpect(jsonPath("$.visible").value(DEFAULT_VISIBLE.booleanValue()));
    }

    @Test
    @Transactional
    void getNonExistingQuestionAnswer() throws Exception {
        // Get the questionAnswer
        restQuestionAnswerMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingQuestionAnswer() throws Exception {
        // Initialize the database
        insertedQuestionAnswer = questionAnswerRepository.saveAndFlush(questionAnswer);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionAnswer
        QuestionAnswer updatedQuestionAnswer = questionAnswerRepository.findById(questionAnswer.getId()).orElseThrow();
        // Disconnect from session so that the updates on updatedQuestionAnswer are not directly saved in db
        em.detach(updatedQuestionAnswer);
        updatedQuestionAnswer.answer(UPDATED_ANSWER).value(UPDATED_VALUE).visible(UPDATED_VISIBLE);

        restQuestionAnswerMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedQuestionAnswer.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedQuestionAnswer))
            )
            .andExpect(status().isOk());

        // Validate the QuestionAnswer in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedQuestionAnswerToMatchAllProperties(updatedQuestionAnswer);
    }

    @Test
    @Transactional
    void putNonExistingQuestionAnswer() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAnswer.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restQuestionAnswerMockMvc
            .perform(
                put(ENTITY_API_URL_ID, questionAnswer.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionAnswer))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionAnswer in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchQuestionAnswer() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAnswer.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionAnswerMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionAnswer))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionAnswer in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamQuestionAnswer() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAnswer.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionAnswerMockMvc
            .perform(put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(questionAnswer)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the QuestionAnswer in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateQuestionAnswerWithPatch() throws Exception {
        // Initialize the database
        insertedQuestionAnswer = questionAnswerRepository.saveAndFlush(questionAnswer);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionAnswer using partial update
        QuestionAnswer partialUpdatedQuestionAnswer = new QuestionAnswer();
        partialUpdatedQuestionAnswer.setId(questionAnswer.getId());

        partialUpdatedQuestionAnswer.visible(UPDATED_VISIBLE);

        restQuestionAnswerMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedQuestionAnswer.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedQuestionAnswer))
            )
            .andExpect(status().isOk());

        // Validate the QuestionAnswer in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertQuestionAnswerUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedQuestionAnswer, questionAnswer),
            getPersistedQuestionAnswer(questionAnswer)
        );
    }

    @Test
    @Transactional
    void fullUpdateQuestionAnswerWithPatch() throws Exception {
        // Initialize the database
        insertedQuestionAnswer = questionAnswerRepository.saveAndFlush(questionAnswer);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionAnswer using partial update
        QuestionAnswer partialUpdatedQuestionAnswer = new QuestionAnswer();
        partialUpdatedQuestionAnswer.setId(questionAnswer.getId());

        partialUpdatedQuestionAnswer.answer(UPDATED_ANSWER).value(UPDATED_VALUE).visible(UPDATED_VISIBLE);

        restQuestionAnswerMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedQuestionAnswer.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedQuestionAnswer))
            )
            .andExpect(status().isOk());

        // Validate the QuestionAnswer in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertQuestionAnswerUpdatableFieldsEquals(partialUpdatedQuestionAnswer, getPersistedQuestionAnswer(partialUpdatedQuestionAnswer));
    }

    @Test
    @Transactional
    void patchNonExistingQuestionAnswer() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAnswer.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restQuestionAnswerMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, questionAnswer.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(questionAnswer))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionAnswer in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchQuestionAnswer() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAnswer.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionAnswerMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(questionAnswer))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionAnswer in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamQuestionAnswer() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAnswer.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionAnswerMockMvc
            .perform(
                patch(ENTITY_API_URL).with(csrf()).contentType("application/merge-patch+json").content(om.writeValueAsBytes(questionAnswer))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the QuestionAnswer in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteQuestionAnswer() throws Exception {
        // Initialize the database
        insertedQuestionAnswer = questionAnswerRepository.saveAndFlush(questionAnswer);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the questionAnswer
        restQuestionAnswerMockMvc
            .perform(delete(ENTITY_API_URL_ID, questionAnswer.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return questionAnswerRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected QuestionAnswer getPersistedQuestionAnswer(QuestionAnswer questionAnswer) {
        return questionAnswerRepository.findById(questionAnswer.getId()).orElseThrow();
    }

    protected void assertPersistedQuestionAnswerToMatchAllProperties(QuestionAnswer expectedQuestionAnswer) {
        assertQuestionAnswerAllPropertiesEquals(expectedQuestionAnswer, getPersistedQuestionAnswer(expectedQuestionAnswer));
    }

    protected void assertPersistedQuestionAnswerToMatchUpdatableProperties(QuestionAnswer expectedQuestionAnswer) {
        assertQuestionAnswerAllUpdatablePropertiesEquals(expectedQuestionAnswer, getPersistedQuestionAnswer(expectedQuestionAnswer));
    }
}
