package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.QuestionAssuranceLevelAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.QuestionAssuranceLevel;
import com.emerald.rcm.backend.repository.QuestionAssuranceLevelRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link QuestionAssuranceLevelResource} REST controller.
 */
@IntegrationTest
@AutoConfigureMockMvc
@WithMockUser
class QuestionAssuranceLevelResourceIT {

    private static final String DEFAULT_ASSURANCE_LEVEL = "AAAAAAAAAA";
    private static final String UPDATED_ASSURANCE_LEVEL = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/question-assurance-levels";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private QuestionAssuranceLevelRepository questionAssuranceLevelRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restQuestionAssuranceLevelMockMvc;

    private QuestionAssuranceLevel questionAssuranceLevel;

    private QuestionAssuranceLevel insertedQuestionAssuranceLevel;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static QuestionAssuranceLevel createEntity(EntityManager em) {
        QuestionAssuranceLevel questionAssuranceLevel = new QuestionAssuranceLevel().assuranceLevel(DEFAULT_ASSURANCE_LEVEL);
        return questionAssuranceLevel;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static QuestionAssuranceLevel createUpdatedEntity(EntityManager em) {
        QuestionAssuranceLevel questionAssuranceLevel = new QuestionAssuranceLevel().assuranceLevel(UPDATED_ASSURANCE_LEVEL);
        return questionAssuranceLevel;
    }

    @BeforeEach
    public void initTest() {
        questionAssuranceLevel = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedQuestionAssuranceLevel != null) {
            questionAssuranceLevelRepository.delete(insertedQuestionAssuranceLevel);
            insertedQuestionAssuranceLevel = null;
        }
    }

    @Test
    @Transactional
    void createQuestionAssuranceLevel() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the QuestionAssuranceLevel
        var returnedQuestionAssuranceLevel = om.readValue(
            restQuestionAssuranceLevelMockMvc
                .perform(
                    post(ENTITY_API_URL)
                        .with(csrf())
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(om.writeValueAsBytes(questionAssuranceLevel))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            QuestionAssuranceLevel.class
        );

        // Validate the QuestionAssuranceLevel in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertQuestionAssuranceLevelUpdatableFieldsEquals(
            returnedQuestionAssuranceLevel,
            getPersistedQuestionAssuranceLevel(returnedQuestionAssuranceLevel)
        );

        insertedQuestionAssuranceLevel = returnedQuestionAssuranceLevel;
    }

    @Test
    @Transactional
    void createQuestionAssuranceLevelWithExistingId() throws Exception {
        // Create the QuestionAssuranceLevel with an existing ID
        questionAssuranceLevel.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restQuestionAssuranceLevelMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionAssuranceLevel))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionAssuranceLevel in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkAssuranceLevelIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        questionAssuranceLevel.setAssuranceLevel(null);

        // Create the QuestionAssuranceLevel, which fails.

        restQuestionAssuranceLevelMockMvc
            .perform(
                post(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionAssuranceLevel))
            )
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllQuestionAssuranceLevels() throws Exception {
        // Initialize the database
        insertedQuestionAssuranceLevel = questionAssuranceLevelRepository.saveAndFlush(questionAssuranceLevel);

        // Get all the questionAssuranceLevelList
        restQuestionAssuranceLevelMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(questionAssuranceLevel.getId().intValue())))
            .andExpect(jsonPath("$.[*].assuranceLevel").value(hasItem(DEFAULT_ASSURANCE_LEVEL)));
    }

    @Test
    @Transactional
    void getQuestionAssuranceLevel() throws Exception {
        // Initialize the database
        insertedQuestionAssuranceLevel = questionAssuranceLevelRepository.saveAndFlush(questionAssuranceLevel);

        // Get the questionAssuranceLevel
        restQuestionAssuranceLevelMockMvc
            .perform(get(ENTITY_API_URL_ID, questionAssuranceLevel.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(questionAssuranceLevel.getId().intValue()))
            .andExpect(jsonPath("$.assuranceLevel").value(DEFAULT_ASSURANCE_LEVEL));
    }

    @Test
    @Transactional
    void getNonExistingQuestionAssuranceLevel() throws Exception {
        // Get the questionAssuranceLevel
        restQuestionAssuranceLevelMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingQuestionAssuranceLevel() throws Exception {
        // Initialize the database
        insertedQuestionAssuranceLevel = questionAssuranceLevelRepository.saveAndFlush(questionAssuranceLevel);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionAssuranceLevel
        QuestionAssuranceLevel updatedQuestionAssuranceLevel = questionAssuranceLevelRepository
            .findById(questionAssuranceLevel.getId())
            .orElseThrow();
        // Disconnect from session so that the updates on updatedQuestionAssuranceLevel are not directly saved in db
        em.detach(updatedQuestionAssuranceLevel);
        updatedQuestionAssuranceLevel.assuranceLevel(UPDATED_ASSURANCE_LEVEL);

        restQuestionAssuranceLevelMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedQuestionAssuranceLevel.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedQuestionAssuranceLevel))
            )
            .andExpect(status().isOk());

        // Validate the QuestionAssuranceLevel in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedQuestionAssuranceLevelToMatchAllProperties(updatedQuestionAssuranceLevel);
    }

    @Test
    @Transactional
    void putNonExistingQuestionAssuranceLevel() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAssuranceLevel.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restQuestionAssuranceLevelMockMvc
            .perform(
                put(ENTITY_API_URL_ID, questionAssuranceLevel.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionAssuranceLevel))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionAssuranceLevel in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchQuestionAssuranceLevel() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAssuranceLevel.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionAssuranceLevelMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionAssuranceLevel))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionAssuranceLevel in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamQuestionAssuranceLevel() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAssuranceLevel.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionAssuranceLevelMockMvc
            .perform(
                put(ENTITY_API_URL)
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(questionAssuranceLevel))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the QuestionAssuranceLevel in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateQuestionAssuranceLevelWithPatch() throws Exception {
        // Initialize the database
        insertedQuestionAssuranceLevel = questionAssuranceLevelRepository.saveAndFlush(questionAssuranceLevel);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionAssuranceLevel using partial update
        QuestionAssuranceLevel partialUpdatedQuestionAssuranceLevel = new QuestionAssuranceLevel();
        partialUpdatedQuestionAssuranceLevel.setId(questionAssuranceLevel.getId());

        restQuestionAssuranceLevelMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedQuestionAssuranceLevel.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedQuestionAssuranceLevel))
            )
            .andExpect(status().isOk());

        // Validate the QuestionAssuranceLevel in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertQuestionAssuranceLevelUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedQuestionAssuranceLevel, questionAssuranceLevel),
            getPersistedQuestionAssuranceLevel(questionAssuranceLevel)
        );
    }

    @Test
    @Transactional
    void fullUpdateQuestionAssuranceLevelWithPatch() throws Exception {
        // Initialize the database
        insertedQuestionAssuranceLevel = questionAssuranceLevelRepository.saveAndFlush(questionAssuranceLevel);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the questionAssuranceLevel using partial update
        QuestionAssuranceLevel partialUpdatedQuestionAssuranceLevel = new QuestionAssuranceLevel();
        partialUpdatedQuestionAssuranceLevel.setId(questionAssuranceLevel.getId());

        partialUpdatedQuestionAssuranceLevel.assuranceLevel(UPDATED_ASSURANCE_LEVEL);

        restQuestionAssuranceLevelMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedQuestionAssuranceLevel.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedQuestionAssuranceLevel))
            )
            .andExpect(status().isOk());

        // Validate the QuestionAssuranceLevel in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertQuestionAssuranceLevelUpdatableFieldsEquals(
            partialUpdatedQuestionAssuranceLevel,
            getPersistedQuestionAssuranceLevel(partialUpdatedQuestionAssuranceLevel)
        );
    }

    @Test
    @Transactional
    void patchNonExistingQuestionAssuranceLevel() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAssuranceLevel.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restQuestionAssuranceLevelMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, questionAssuranceLevel.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(questionAssuranceLevel))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionAssuranceLevel in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchQuestionAssuranceLevel() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAssuranceLevel.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionAssuranceLevelMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(questionAssuranceLevel))
            )
            .andExpect(status().isBadRequest());

        // Validate the QuestionAssuranceLevel in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamQuestionAssuranceLevel() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        questionAssuranceLevel.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restQuestionAssuranceLevelMockMvc
            .perform(
                patch(ENTITY_API_URL)
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(questionAssuranceLevel))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the QuestionAssuranceLevel in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteQuestionAssuranceLevel() throws Exception {
        // Initialize the database
        insertedQuestionAssuranceLevel = questionAssuranceLevelRepository.saveAndFlush(questionAssuranceLevel);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the questionAssuranceLevel
        restQuestionAssuranceLevelMockMvc
            .perform(delete(ENTITY_API_URL_ID, questionAssuranceLevel.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return questionAssuranceLevelRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected QuestionAssuranceLevel getPersistedQuestionAssuranceLevel(QuestionAssuranceLevel questionAssuranceLevel) {
        return questionAssuranceLevelRepository.findById(questionAssuranceLevel.getId()).orElseThrow();
    }

    protected void assertPersistedQuestionAssuranceLevelToMatchAllProperties(QuestionAssuranceLevel expectedQuestionAssuranceLevel) {
        assertQuestionAssuranceLevelAllPropertiesEquals(
            expectedQuestionAssuranceLevel,
            getPersistedQuestionAssuranceLevel(expectedQuestionAssuranceLevel)
        );
    }

    protected void assertPersistedQuestionAssuranceLevelToMatchUpdatableProperties(QuestionAssuranceLevel expectedQuestionAssuranceLevel) {
        assertQuestionAssuranceLevelAllUpdatablePropertiesEquals(
            expectedQuestionAssuranceLevel,
            getPersistedQuestionAssuranceLevel(expectedQuestionAssuranceLevel)
        );
    }
}
