package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.ResourceTypeAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.ResourceType;
import com.emerald.rcm.backend.repository.ResourceTypeRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link ResourceTypeResource} REST controller.
 */
@IntegrationTest
@AutoConfigureMockMvc
@WithMockUser
class ResourceTypeResourceIT {

    private static final String DEFAULT_NAME = "AAAAAAAAAA";
    private static final String UPDATED_NAME = "BBBBBBBBBB";

    private static final String DEFAULT_ONTOLOGY_TERM = "AAAAAAAAAA";
    private static final String UPDATED_ONTOLOGY_TERM = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/resource-types";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private ResourceTypeRepository resourceTypeRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restResourceTypeMockMvc;

    private ResourceType resourceType;

    private ResourceType insertedResourceType;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static ResourceType createEntity(EntityManager em) {
        ResourceType resourceType = new ResourceType().name(DEFAULT_NAME).ontologyTerm(DEFAULT_ONTOLOGY_TERM);
        return resourceType;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static ResourceType createUpdatedEntity(EntityManager em) {
        ResourceType resourceType = new ResourceType().name(UPDATED_NAME).ontologyTerm(UPDATED_ONTOLOGY_TERM);
        return resourceType;
    }

    @BeforeEach
    public void initTest() {
        resourceType = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedResourceType != null) {
            resourceTypeRepository.delete(insertedResourceType);
            insertedResourceType = null;
        }
    }

    @Test
    @Transactional
    void createResourceType() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the ResourceType
        var returnedResourceType = om.readValue(
            restResourceTypeMockMvc
                .perform(
                    post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(resourceType))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            ResourceType.class
        );

        // Validate the ResourceType in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertResourceTypeUpdatableFieldsEquals(returnedResourceType, getPersistedResourceType(returnedResourceType));

        insertedResourceType = returnedResourceType;
    }

    @Test
    @Transactional
    void createResourceTypeWithExistingId() throws Exception {
        // Create the ResourceType with an existing ID
        resourceType.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restResourceTypeMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(resourceType)))
            .andExpect(status().isBadRequest());

        // Validate the ResourceType in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkNameIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        resourceType.setName(null);

        // Create the ResourceType, which fails.

        restResourceTypeMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(resourceType)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void checkOntologyTermIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        resourceType.setOntologyTerm(null);

        // Create the ResourceType, which fails.

        restResourceTypeMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(resourceType)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllResourceTypes() throws Exception {
        // Initialize the database
        insertedResourceType = resourceTypeRepository.saveAndFlush(resourceType);

        // Get all the resourceTypeList
        restResourceTypeMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(resourceType.getId().intValue())))
            .andExpect(jsonPath("$.[*].name").value(hasItem(DEFAULT_NAME)))
            .andExpect(jsonPath("$.[*].ontologyTerm").value(hasItem(DEFAULT_ONTOLOGY_TERM)));
    }

    @Test
    @Transactional
    void getResourceType() throws Exception {
        // Initialize the database
        insertedResourceType = resourceTypeRepository.saveAndFlush(resourceType);

        // Get the resourceType
        restResourceTypeMockMvc
            .perform(get(ENTITY_API_URL_ID, resourceType.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(resourceType.getId().intValue()))
            .andExpect(jsonPath("$.name").value(DEFAULT_NAME))
            .andExpect(jsonPath("$.ontologyTerm").value(DEFAULT_ONTOLOGY_TERM));
    }

    @Test
    @Transactional
    void getNonExistingResourceType() throws Exception {
        // Get the resourceType
        restResourceTypeMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingResourceType() throws Exception {
        // Initialize the database
        insertedResourceType = resourceTypeRepository.saveAndFlush(resourceType);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the resourceType
        ResourceType updatedResourceType = resourceTypeRepository.findById(resourceType.getId()).orElseThrow();
        // Disconnect from session so that the updates on updatedResourceType are not directly saved in db
        em.detach(updatedResourceType);
        updatedResourceType.name(UPDATED_NAME).ontologyTerm(UPDATED_ONTOLOGY_TERM);

        restResourceTypeMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedResourceType.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedResourceType))
            )
            .andExpect(status().isOk());

        // Validate the ResourceType in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedResourceTypeToMatchAllProperties(updatedResourceType);
    }

    @Test
    @Transactional
    void putNonExistingResourceType() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        resourceType.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restResourceTypeMockMvc
            .perform(
                put(ENTITY_API_URL_ID, resourceType.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(resourceType))
            )
            .andExpect(status().isBadRequest());

        // Validate the ResourceType in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchResourceType() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        resourceType.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restResourceTypeMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(resourceType))
            )
            .andExpect(status().isBadRequest());

        // Validate the ResourceType in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamResourceType() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        resourceType.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restResourceTypeMockMvc
            .perform(put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(resourceType)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the ResourceType in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateResourceTypeWithPatch() throws Exception {
        // Initialize the database
        insertedResourceType = resourceTypeRepository.saveAndFlush(resourceType);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the resourceType using partial update
        ResourceType partialUpdatedResourceType = new ResourceType();
        partialUpdatedResourceType.setId(resourceType.getId());

        partialUpdatedResourceType.ontologyTerm(UPDATED_ONTOLOGY_TERM);

        restResourceTypeMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedResourceType.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedResourceType))
            )
            .andExpect(status().isOk());

        // Validate the ResourceType in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertResourceTypeUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedResourceType, resourceType),
            getPersistedResourceType(resourceType)
        );
    }

    @Test
    @Transactional
    void fullUpdateResourceTypeWithPatch() throws Exception {
        // Initialize the database
        insertedResourceType = resourceTypeRepository.saveAndFlush(resourceType);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the resourceType using partial update
        ResourceType partialUpdatedResourceType = new ResourceType();
        partialUpdatedResourceType.setId(resourceType.getId());

        partialUpdatedResourceType.name(UPDATED_NAME).ontologyTerm(UPDATED_ONTOLOGY_TERM);

        restResourceTypeMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedResourceType.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedResourceType))
            )
            .andExpect(status().isOk());

        // Validate the ResourceType in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertResourceTypeUpdatableFieldsEquals(partialUpdatedResourceType, getPersistedResourceType(partialUpdatedResourceType));
    }

    @Test
    @Transactional
    void patchNonExistingResourceType() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        resourceType.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restResourceTypeMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, resourceType.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(resourceType))
            )
            .andExpect(status().isBadRequest());

        // Validate the ResourceType in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchResourceType() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        resourceType.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restResourceTypeMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(resourceType))
            )
            .andExpect(status().isBadRequest());

        // Validate the ResourceType in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamResourceType() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        resourceType.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restResourceTypeMockMvc
            .perform(
                patch(ENTITY_API_URL).with(csrf()).contentType("application/merge-patch+json").content(om.writeValueAsBytes(resourceType))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the ResourceType in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteResourceType() throws Exception {
        // Initialize the database
        insertedResourceType = resourceTypeRepository.saveAndFlush(resourceType);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the resourceType
        restResourceTypeMockMvc
            .perform(delete(ENTITY_API_URL_ID, resourceType.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return resourceTypeRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected ResourceType getPersistedResourceType(ResourceType resourceType) {
        return resourceTypeRepository.findById(resourceType.getId()).orElseThrow();
    }

    protected void assertPersistedResourceTypeToMatchAllProperties(ResourceType expectedResourceType) {
        assertResourceTypeAllPropertiesEquals(expectedResourceType, getPersistedResourceType(expectedResourceType));
    }

    protected void assertPersistedResourceTypeToMatchUpdatableProperties(ResourceType expectedResourceType) {
        assertResourceTypeAllUpdatablePropertiesEquals(expectedResourceType, getPersistedResourceType(expectedResourceType));
    }
}
