package com.emerald.rcm.backend.web.rest;

import static com.emerald.rcm.backend.domain.TargetValueAsserts.*;
import static com.emerald.rcm.backend.web.rest.TestUtil.createUpdateProxyForBean;
import static org.assertj.core.api.Assertions.assertThat;
import static org.hamcrest.Matchers.hasItem;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.csrf;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import com.emerald.rcm.backend.IntegrationTest;
import com.emerald.rcm.backend.domain.TargetValue;
import com.emerald.rcm.backend.repository.TargetValueRepository;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.persistence.EntityManager;
import java.util.Random;
import java.util.concurrent.atomic.AtomicLong;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.transaction.annotation.Transactional;

/**
 * Integration tests for the {@link TargetValueResource} REST controller.
 */
@IntegrationTest
@AutoConfigureMockMvc
@WithMockUser
class TargetValueResourceIT {

    private static final String DEFAULT_VALUE = "AAAAAAAAAA";
    private static final String UPDATED_VALUE = "BBBBBBBBBB";

    private static final String ENTITY_API_URL = "/api/target-values";
    private static final String ENTITY_API_URL_ID = ENTITY_API_URL + "/{id}";

    private static Random random = new Random();
    private static AtomicLong longCount = new AtomicLong(random.nextInt() + (2 * Integer.MAX_VALUE));

    @Autowired
    private ObjectMapper om;

    @Autowired
    private TargetValueRepository targetValueRepository;

    @Autowired
    private EntityManager em;

    @Autowired
    private MockMvc restTargetValueMockMvc;

    private TargetValue targetValue;

    private TargetValue insertedTargetValue;

    /**
     * Create an entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static TargetValue createEntity(EntityManager em) {
        TargetValue targetValue = new TargetValue().value(DEFAULT_VALUE);
        return targetValue;
    }

    /**
     * Create an updated entity for this test.
     *
     * This is a static method, as tests for other entities might also need it,
     * if they test an entity which requires the current entity.
     */
    public static TargetValue createUpdatedEntity(EntityManager em) {
        TargetValue targetValue = new TargetValue().value(UPDATED_VALUE);
        return targetValue;
    }

    @BeforeEach
    public void initTest() {
        targetValue = createEntity(em);
    }

    @AfterEach
    public void cleanup() {
        if (insertedTargetValue != null) {
            targetValueRepository.delete(insertedTargetValue);
            insertedTargetValue = null;
        }
    }

    @Test
    @Transactional
    void createTargetValue() throws Exception {
        long databaseSizeBeforeCreate = getRepositoryCount();
        // Create the TargetValue
        var returnedTargetValue = om.readValue(
            restTargetValueMockMvc
                .perform(
                    post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(targetValue))
                )
                .andExpect(status().isCreated())
                .andReturn()
                .getResponse()
                .getContentAsString(),
            TargetValue.class
        );

        // Validate the TargetValue in the database
        assertIncrementedRepositoryCount(databaseSizeBeforeCreate);
        assertTargetValueUpdatableFieldsEquals(returnedTargetValue, getPersistedTargetValue(returnedTargetValue));

        insertedTargetValue = returnedTargetValue;
    }

    @Test
    @Transactional
    void createTargetValueWithExistingId() throws Exception {
        // Create the TargetValue with an existing ID
        targetValue.setId(1L);

        long databaseSizeBeforeCreate = getRepositoryCount();

        // An entity with an existing ID cannot be created, so this API call must fail
        restTargetValueMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(targetValue)))
            .andExpect(status().isBadRequest());

        // Validate the TargetValue in the database
        assertSameRepositoryCount(databaseSizeBeforeCreate);
    }

    @Test
    @Transactional
    void checkValueIsRequired() throws Exception {
        long databaseSizeBeforeTest = getRepositoryCount();
        // set the field null
        targetValue.setValue(null);

        // Create the TargetValue, which fails.

        restTargetValueMockMvc
            .perform(post(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(targetValue)))
            .andExpect(status().isBadRequest());

        assertSameRepositoryCount(databaseSizeBeforeTest);
    }

    @Test
    @Transactional
    void getAllTargetValues() throws Exception {
        // Initialize the database
        insertedTargetValue = targetValueRepository.saveAndFlush(targetValue);

        // Get all the targetValueList
        restTargetValueMockMvc
            .perform(get(ENTITY_API_URL + "?sort=id,desc"))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.[*].id").value(hasItem(targetValue.getId().intValue())))
            .andExpect(jsonPath("$.[*].value").value(hasItem(DEFAULT_VALUE)));
    }

    @Test
    @Transactional
    void getTargetValue() throws Exception {
        // Initialize the database
        insertedTargetValue = targetValueRepository.saveAndFlush(targetValue);

        // Get the targetValue
        restTargetValueMockMvc
            .perform(get(ENTITY_API_URL_ID, targetValue.getId()))
            .andExpect(status().isOk())
            .andExpect(content().contentType(MediaType.APPLICATION_JSON_VALUE))
            .andExpect(jsonPath("$.id").value(targetValue.getId().intValue()))
            .andExpect(jsonPath("$.value").value(DEFAULT_VALUE));
    }

    @Test
    @Transactional
    void getNonExistingTargetValue() throws Exception {
        // Get the targetValue
        restTargetValueMockMvc.perform(get(ENTITY_API_URL_ID, Long.MAX_VALUE)).andExpect(status().isNotFound());
    }

    @Test
    @Transactional
    void putExistingTargetValue() throws Exception {
        // Initialize the database
        insertedTargetValue = targetValueRepository.saveAndFlush(targetValue);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the targetValue
        TargetValue updatedTargetValue = targetValueRepository.findById(targetValue.getId()).orElseThrow();
        // Disconnect from session so that the updates on updatedTargetValue are not directly saved in db
        em.detach(updatedTargetValue);
        updatedTargetValue.value(UPDATED_VALUE);

        restTargetValueMockMvc
            .perform(
                put(ENTITY_API_URL_ID, updatedTargetValue.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(updatedTargetValue))
            )
            .andExpect(status().isOk());

        // Validate the TargetValue in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertPersistedTargetValueToMatchAllProperties(updatedTargetValue);
    }

    @Test
    @Transactional
    void putNonExistingTargetValue() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        targetValue.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restTargetValueMockMvc
            .perform(
                put(ENTITY_API_URL_ID, targetValue.getId())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(targetValue))
            )
            .andExpect(status().isBadRequest());

        // Validate the TargetValue in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithIdMismatchTargetValue() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        targetValue.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restTargetValueMockMvc
            .perform(
                put(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType(MediaType.APPLICATION_JSON)
                    .content(om.writeValueAsBytes(targetValue))
            )
            .andExpect(status().isBadRequest());

        // Validate the TargetValue in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void putWithMissingIdPathParamTargetValue() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        targetValue.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restTargetValueMockMvc
            .perform(put(ENTITY_API_URL).with(csrf()).contentType(MediaType.APPLICATION_JSON).content(om.writeValueAsBytes(targetValue)))
            .andExpect(status().isMethodNotAllowed());

        // Validate the TargetValue in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void partialUpdateTargetValueWithPatch() throws Exception {
        // Initialize the database
        insertedTargetValue = targetValueRepository.saveAndFlush(targetValue);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the targetValue using partial update
        TargetValue partialUpdatedTargetValue = new TargetValue();
        partialUpdatedTargetValue.setId(targetValue.getId());

        restTargetValueMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedTargetValue.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedTargetValue))
            )
            .andExpect(status().isOk());

        // Validate the TargetValue in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertTargetValueUpdatableFieldsEquals(
            createUpdateProxyForBean(partialUpdatedTargetValue, targetValue),
            getPersistedTargetValue(targetValue)
        );
    }

    @Test
    @Transactional
    void fullUpdateTargetValueWithPatch() throws Exception {
        // Initialize the database
        insertedTargetValue = targetValueRepository.saveAndFlush(targetValue);

        long databaseSizeBeforeUpdate = getRepositoryCount();

        // Update the targetValue using partial update
        TargetValue partialUpdatedTargetValue = new TargetValue();
        partialUpdatedTargetValue.setId(targetValue.getId());

        partialUpdatedTargetValue.value(UPDATED_VALUE);

        restTargetValueMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, partialUpdatedTargetValue.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(partialUpdatedTargetValue))
            )
            .andExpect(status().isOk());

        // Validate the TargetValue in the database

        assertSameRepositoryCount(databaseSizeBeforeUpdate);
        assertTargetValueUpdatableFieldsEquals(partialUpdatedTargetValue, getPersistedTargetValue(partialUpdatedTargetValue));
    }

    @Test
    @Transactional
    void patchNonExistingTargetValue() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        targetValue.setId(longCount.incrementAndGet());

        // If the entity doesn't have an ID, it will throw BadRequestAlertException
        restTargetValueMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, targetValue.getId())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(targetValue))
            )
            .andExpect(status().isBadRequest());

        // Validate the TargetValue in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithIdMismatchTargetValue() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        targetValue.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restTargetValueMockMvc
            .perform(
                patch(ENTITY_API_URL_ID, longCount.incrementAndGet())
                    .with(csrf())
                    .contentType("application/merge-patch+json")
                    .content(om.writeValueAsBytes(targetValue))
            )
            .andExpect(status().isBadRequest());

        // Validate the TargetValue in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void patchWithMissingIdPathParamTargetValue() throws Exception {
        long databaseSizeBeforeUpdate = getRepositoryCount();
        targetValue.setId(longCount.incrementAndGet());

        // If url ID doesn't match entity ID, it will throw BadRequestAlertException
        restTargetValueMockMvc
            .perform(
                patch(ENTITY_API_URL).with(csrf()).contentType("application/merge-patch+json").content(om.writeValueAsBytes(targetValue))
            )
            .andExpect(status().isMethodNotAllowed());

        // Validate the TargetValue in the database
        assertSameRepositoryCount(databaseSizeBeforeUpdate);
    }

    @Test
    @Transactional
    void deleteTargetValue() throws Exception {
        // Initialize the database
        insertedTargetValue = targetValueRepository.saveAndFlush(targetValue);

        long databaseSizeBeforeDelete = getRepositoryCount();

        // Delete the targetValue
        restTargetValueMockMvc
            .perform(delete(ENTITY_API_URL_ID, targetValue.getId()).with(csrf()).accept(MediaType.APPLICATION_JSON))
            .andExpect(status().isNoContent());

        // Validate the database contains one less item
        assertDecrementedRepositoryCount(databaseSizeBeforeDelete);
    }

    protected long getRepositoryCount() {
        return targetValueRepository.count();
    }

    protected void assertIncrementedRepositoryCount(long countBefore) {
        assertThat(countBefore + 1).isEqualTo(getRepositoryCount());
    }

    protected void assertDecrementedRepositoryCount(long countBefore) {
        assertThat(countBefore - 1).isEqualTo(getRepositoryCount());
    }

    protected void assertSameRepositoryCount(long countBefore) {
        assertThat(countBefore).isEqualTo(getRepositoryCount());
    }

    protected TargetValue getPersistedTargetValue(TargetValue targetValue) {
        return targetValueRepository.findById(targetValue.getId()).orElseThrow();
    }

    protected void assertPersistedTargetValueToMatchAllProperties(TargetValue expectedTargetValue) {
        assertTargetValueAllPropertiesEquals(expectedTargetValue, getPersistedTargetValue(expectedTargetValue));
    }

    protected void assertPersistedTargetValueToMatchUpdatableProperties(TargetValue expectedTargetValue) {
        assertTargetValueAllUpdatablePropertiesEquals(expectedTargetValue, getPersistedTargetValue(expectedTargetValue));
    }
}
